package mic1.gui;
/*
 * @(#)ConsoleFrame.java	1.01 12/Dec/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.01 12/Dec/2000
 * @Author  : Richard Klos
 *
 * Description : This is a console for both character input and output.
 *               When a user presses a key, the character will not be
 *               shown in the text area, but is sent to some receiver.
 *               Characters will only be written to the text area when
 *               the writeChar(char) method is called.
 *
 *               IMPORTANT NOTE: A console should be constructed
 *               during the initialisation of the application, made
 *               visible when it is needed and made invisible when
 *               it is no longer needed. This way, the console can
 *               always receive characters, even when it is not
 *               visible.
 *               
 * 
 * Filter for file types
 * 
***  

To Do:
	
Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
Richard Klos     05.12.2000 Created
Pierre Bakker	 09.12.2000	Always on top property set
Richard Klos     12.12.2000 Added a dummy component as KeyListener.

*/

import mic1.ijvm.KeyBuffer;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

public class ConsoleFrame extends JDialog implements KeyListener, ActionListener, FocusListener
{
	private JTextArea taView;
	private JButton bClear, bClose;
	private MainFrame m_parent;
	private JPanel dummy;

	/**
	 * Constructor.
	 * If the key buffer is null, the console will not
	 * enqueue input from the keyboard.
	 */	
	public ConsoleFrame(MainFrame mf)
	{
		super(mf, false);
		m_parent = mf;
		setTitle("Console");
		setSize(350,250);

		// Place in the center.
		Dimension screen = getToolkit().getScreenSize();
		setLocation((screen.width - getSize().width) / 2, (screen.height - getSize().height) / 2);
		
		// Create and configure components.
		bClear = new JButton("Clear");
		bClear.addActionListener(this);
		bClear.addFocusListener(this);
		
		bClose = new JButton("Close");
		bClose.addActionListener(this);
		bClose.addFocusListener(this);
		
		taView = new JTextArea();
		taView.setFont(new Font("Monospaced", Font.PLAIN, 14));
		taView.setLineWrap(true);
		taView.setWrapStyleWord(true);
		taView.setEditable(false);
		taView.addFocusListener(this);
		taView.addKeyListener(this);
		
		// Create dummy component.
		dummy = new JPanel();
		dummy.addKeyListener(this);
		dummy.addFocusListener(this);

		// Build bottom panel.
		JPanel pBottom = new JPanel();
		pBottom.setLayout(new FlowLayout(FlowLayout.RIGHT, 30, 5));
		pBottom.add(dummy);
		pBottom.add(bClear);
		pBottom.add(bClose);
		
		// Build center panel.
		JScrollPane scrollPane = new JScrollPane(taView,
		                                         JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
		                                         JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		Container cp = getContentPane();
		cp.setLayout(new BorderLayout(5, 5));
		cp.add("South", pBottom);
		cp.add("Center", scrollPane);
	
		setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
	}

	/**
	 * Clears the console screen.
	 */
	public void clear()	
	{
		taView.setText(null);
	}

	
	/**
	 * Writes a character to the screen.
	 * 
	 * @param: character
	 */
	public void writeChar(char c)
	{
		taView.append("" + c);
	}


	/**
	 * Closes (hides) this console.
	 */
	private void close()
	{
		setVisible(false);
	}


	// FocusListener (used to keep the focus on the text area)
	public void focusGained(FocusEvent e)
	{
		if(e.getSource() != dummy)
			dummy.requestFocus();
	}
	
	
	public void focusLost(FocusEvent e)
	{
		if(e.getSource() == dummy)
			dummy.requestFocus();
	}
	
	
	// KeyListener	
	public void keyTyped(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}
	public void keyPressed(KeyEvent e)
	{
		int keyCode = e.getKeyCode();
		if(keyCode == KeyEvent.VK_ENTER)
			m_parent.sendCharacter('\n');
		else if(keyCode == KeyEvent.VK_ESCAPE)
			close();
		else
			m_parent.sendCharacter(e.getKeyChar());
	}


	// ActionListener
	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		if(source == bClear)
			clear();
		else if(source == bClose)
			close();
	}
}
