package mic1.gui;
/*
 * @(#)CpuView.java	1.0 05/Dec/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.20 19/Jan/2001
 * @Author  : PB
 *
 * Description : 
 * 
 * Creates the CPU view
 * 
***  

To Do:


Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
Pierre Bakker    05.12.2000 Created
Pierre Bakker	 08.12.2000	Added Simulator functionality
Pierre Bakker	 18.12.2000 enable / disable table editing when running or not
Richard Klos     19.01.2001 Accept #, $, or 0x as valid input in hexadecimal numbers.
Pierre Bakker	 22.01.2001 Cleaned up
Pierre Bakker	 27.06.2001 MIR seperation
*/

import java.awt.*;
import java.awt.event.*;
import java.awt.Image.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JComponent;
import javax.swing.JTable;

import mic1.ijvm.*;
import mic1.util.*;

public class CpuView extends JPanel implements Commands
{
	private MainFrame		m_Parent;
	private RegisterView	m_RegView;
	private FlagView		m_FlagView;
	private JButton			m_Step,
							m_Stop,
							m_Run;	
	
	CpuView(MainFrame caller)	
	{
		m_Parent = caller;

    	setLayout(new BorderLayout());   
    	setBorder(BorderFactory.createTitledBorder("CPU monitor"));         	
    	add(createRegisterView(), BorderLayout.CENTER);
    	add(createControls(), BorderLayout.SOUTH);
	}
	
	public void setEnabled(boolean bState, int id)
	{
		if(id == MyMenuBar.ONRUN_ITEMS)
		{
			m_Step.setEnabled(bState);
			m_Stop.setEnabled(!bState);
			m_Run.setEnabled(bState);
			m_RegView.setEditable(bState);			
		}
		else if(id == MyMenuBar.ONSTEP_ITEMS)
		{
			m_Step.setEnabled(!bState);
			m_Stop.setEnabled(!bState);
			m_Run.setEnabled(!bState);
		}
		else if(id == MyMenuBar.ALL_ITEMS)
			setEnabled(bState, MyMenuBar.ONRUN_ITEMS);
	}

	/**
	 * sets the value of the give'n register id
	 *
	 * @param : value
	 * @param : Register id
	 */
	public void setRegister(int nValue, int id)
	{
		m_RegView.setRegister("0x" + mic1.util.Util.toHex(nValue, 8), id);
	}	


	/**
	 * sets the string representations of the MIR.
	 *
	 * @param : string representation of the current micro instruction.
	 * @param : string representation of the next micro instruction.
	 */
	public void setMIRRegister(String sInstruction, String sNextInstruction)
	{
		m_RegView.setMIRRegister(sInstruction, sNextInstruction);
	}	

	/**
	 * Set the Negative flag true or false
	 *
	 * @param : Flag state
	 */
	public void setNFlag(boolean n)
	{	m_FlagView.setNFlag(n);		}

	/**
	 * Set the Zero flag true or false
	 *
	 * @param : Flag state
	 */
	public void setZFlag(boolean z)
	{	m_FlagView.setZFlag(z);		}	

	/**
	 * Creates a new RegisterView
	 */
	private JPanel createRegisterView()
	{
		m_RegView = new RegisterView(m_Parent);
		m_RegView.add(createFlagView(), BorderLayout.SOUTH);		
		
		return m_RegView;
	}
	
	/**
	 * Creates a new FlagView
	 */
	private JPanel createFlagView()
	{
		m_FlagView = new FlagView(m_Parent);
		
		return m_FlagView;
	}

	/**
	 * Creates the RUN, STOP and STEP Butten Panel
	 */
	private JPanel createControls()
	{
		m_Step = new JButton(STEP.sLabel);
		m_Stop = new JButton(STOP.sLabel);
		m_Run = new JButton(RUN.sLabel);
		
		JPanel controls = new JPanel();
		controls.setLayout(new GridLayout(1,3));
		  controls.add(m_Step);
		  controls.add(m_Stop);
		  controls.add(m_Run);
		  m_Step.addActionListener(m_Parent);
		  m_Stop.addActionListener(m_Parent);
		  m_Run.addActionListener(m_Parent);
					
		return controls;
	}
}



class RegisterView extends JPanel implements Labels, PropertyChangeListener
{
	private final int				m_iColm = 0;					//colm number								
	private static final Color		m_cBgColor = Color.lightGray;
	private JTable 					m_Table;
	private Mic1Interface			m_Simulator;
	private MainFrame 				m_Parent;	

	RegisterView(MainFrame caller)
	{
		super();
		m_Parent = caller;
		m_Simulator = m_Parent.getSimulator();

		setLayout(new BorderLayout());
		m_Table = new JTable(13,1);		
		m_Table.addPropertyChangeListener(this);
		add(m_Table, BorderLayout.CENTER);
	
		JPanel registers = new JPanel()
		{
			int xOffset = 3;
			int yOffset = -3;
			public void paint(Graphics g)
			{
				super.paint(g);
				
				int placement = m_Table.getRowHeight();
				g.setColor(Color.blue);
				g.setFont(getFont());
				g.drawString(MAR.sLabel, xOffset, yOffset + placement);
				g.drawString(MDR.sLabel, xOffset, yOffset + placement*2);
				g.drawString(PC.sLabel , xOffset, yOffset + placement*3);
				g.drawString(MBR.sLabel, xOffset, yOffset + placement*4);
				g.drawString(LV.sLabel , xOffset, yOffset + placement*5);
				g.drawString(CPP.sLabel, xOffset, yOffset + placement*6);				
				g.drawString(TOS.sLabel, xOffset, yOffset + placement*7);
				g.drawString(OPC.sLabel, xOffset, yOffset + placement*8);
				g.drawString(H.sLabel  , xOffset, yOffset + placement*9);
				g.drawString(MIR1.sLabel , xOffset, yOffset + placement*10);
				g.drawString(MIR2.sLabel , xOffset, yOffset + placement*11);
				g.drawString(MPC.sLabel , xOffset, yOffset + placement*12);				
				g.drawString(SP.sLabel , xOffset, yOffset + placement*13);				
			}
		};
		registers.setPreferredSize(new Dimension(40, getHeight()));
		add(registers, BorderLayout.WEST);		
	}
	
	/**
	 * Makes the table editable or not
	 *
	 * @param : state true or false
	 */
	public void setEditable(boolean bState)
	{
		if(m_Table.isEditing())
			m_Table.removeEditor();
		m_Table.setEnabled(bState);
	}
	
	/**
	 * sets the value of the give'n register id
	 *
	 * @param : Register value
	 * @param : Register id
	 */
	public void setRegister(String sValue, int id)
	{
		m_Table.setValueAt(sValue, id, m_iColm);
	}
	
	/**
	 * sets the string representations of the MIR.
	 *
	 * @param : string representation of the current micro instruction.
	 * @param : string representation of the next micro instruction.
	 */
	public void setMIRRegister(String sInstruction, String sNextInstruction)
	{
		m_Table.setValueAt(sInstruction, Registers.MIR1, m_iColm);
		m_Table.setValueAt(sNextInstruction, Registers.MIR2, m_iColm);
	}	

	/**
	 * Refreshes the simulator
	 */
	protected void refresh()
	{
		m_Simulator.refreshOutput();
	}
	
	public void propertyChange(PropertyChangeEvent evt)
	{
		if(m_Table == null)
			return;
		
		int activeRow = m_Table.getEditingRow();		
		if(activeRow < 0)
			return;
		
		//table parser
		String rowString = (String)m_Table.getValueAt(activeRow,0);		
		try
		{
			int rowInt = mic1.util.Util.hexToInt(rowString);
				
			switch(activeRow)
			{
				case 0:	m_Simulator.setRegisterMAR( rowInt );
						break;
				case 1:	m_Simulator.setRegisterMDR( rowInt );
						break;
				case 2:	m_Simulator.setRegisterPC( rowInt );
						break;	
				case 3:	m_Simulator.setRegisterMBR( rowInt );
						break;
				case 4:	m_Simulator.setRegisterLV( rowInt );
						break;
				case 5:	m_Simulator.setRegisterCPP( rowInt );
						break;
				case 6:	m_Simulator.setRegisterTOS( rowInt);
						break;
				case 7:	m_Simulator.setRegisterOPC( rowInt );
						break;
				case 8:	m_Simulator.setRegisterH( rowInt );
						break;
				case 9:	//m_Simulator.setRegisterMIR(((Integer)m_Table.getValueAt(activeRow,0)).intValue() );
						break;
				case 10:	m_Simulator.setRegisterMPC( rowInt );
						break;
				case 11:	m_Simulator.setRegisterSP( rowInt );
						break;
			}
			m_Parent.getOutput().clear();
			m_Table.removeEditor();			
		}
		catch(Exception e)
		{
			getToolkit().beep();
			m_Parent.getOutput().clear();
			m_Parent.getOutput().println(ERR_NOT_A_VALID_VALUE.sLabel);
			m_Table.removeEditor();
			refresh();
		}		
	}
}

class FlagView extends JPanel
{
	private static ImageIcon 		m_Up;
	private static ImageIcon 		m_Down;
	private static String			m_sN 	= "N =";
	private static String			m_sZ 	= "Z =";
	private static boolean			m_bN	= false;
	private static boolean			m_bZ	= false;
	private MainFrame				m_Parent;
	private final int				m_nHeight = 47;
	
	FlagView(MainFrame caller)
	{		
		setPreferredSize(new Dimension(getWidth(), m_nHeight));
		setBorder(BorderFactory.createTitledBorder("Flags"));
	}
	

	/**
	 * Set the Negative flag true or false and repaints the panel
	 *
	 * @param : Flag state
	 */
	public void setNFlag(boolean bN)
	{
		if(m_bN != bN)
		{
			m_bN = bN;
			repaint();
		}
	}


	/**
	 * Set the Zero flag true or false and repaints the panel
	 *
	 * @param : Flag state
	 */
	public void setZFlag(boolean bZ)
	{
		if(m_bZ != bZ)
		{
			m_bZ = bZ;
			repaint();
		}
	}
	
	public void paint(Graphics g)
	{
		super.paint(g);
		g.setColor(Color.blue);
		g.drawString(m_sZ+m_bZ, 23, m_nHeight-15);
		g.drawString(m_sN+m_bN, 83, m_nHeight-15);		
	}
}
