package mic1.gui;
/*
 * @(#)CustomizeFrame.java	1.01 29/Dec/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.0 29/Dec/2000
 * @Author  : Arno Paanstra
 *
 * Description : 
 * 
 * Dialog for changing the color settings and font types of the editor
 * 
***  

To Do:
	
Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
Arno Paanstra    28.12.2000 Created
Arno Paanstra    29.12.2000 Add color customizing
Arno Paanstra    31.12.2000 Add default and cancel button
Arno Paanstra    12.01.2001 Fix bug: Nullpointer Exception after clicking on cancel in the colorchooser
Arno Paanstra    13.01.2001 Fix bug: Displayed size in Customize panel one too big
Arno Paanstra    14.01.2001 Add printer font customizing
Pierre Bakker	 22.01.2001 Cleaned up

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import mic1.gui.editor.*;

public class CustomizeFrame extends JDialog implements ActionListener
{
	PropertiesManager	m_PropertiesManager;
	
	JComboBox 			m_style_Combobox,
						m_size_Combobox,
						m_printer_style_Combobox,
						m_printer_size_Combobox;
	JButton				m_ok_Button,
						m_cancel_Button,
						m_comment_Button,
						m_literal_Button,
						m_invalid_Button;
	
	public CustomizeFrame(MainFrame mf, PropertiesManager pm )
	{
		super(mf, true);
		m_PropertiesManager = pm;
		setTitle("Customize");
		setSize(450,250);

		// Place in the center.
		Dimension screen = getToolkit().getScreenSize();
		setLocation((screen.width - getSize().width) / 2, (screen.height - getSize().height) / 2);
		
		//Create Customize panel
		JPanel gridPanel = new JPanel();
		gridPanel.setLayout(new GridLayout(1,3));
		
		//Create Editor Customize Panel		
		JPanel ecPanel = new JPanel();
		ecPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Editor Font" ));
		
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		ecPanel.setLayout( gridbag );
		
		JLabel e_select_Label = new JLabel("Select:");
			
		String[] s = new String[] {"Arial","Courier New","Dialog","SansSerif","Serif","Monospaced"};
		JLabel e_font_Label = new JLabel("font");
		m_style_Combobox = new JComboBox( s );
		
		c.gridwidth	= 2;
		c.weightx	= 0.1;	c.weighty	= 0.1;
		c.gridx		= 0;	c.gridy		= 0;
		gridbag.setConstraints(e_select_Label, c);
		ecPanel.add(e_select_Label);
		
		c.gridwidth	= 1;
		c.gridx		= 0;	c.gridy		= 1;
		gridbag.setConstraints(e_font_Label, c);
		ecPanel.add(e_font_Label);
			
		c.gridx		= 1;	c.gridy		= 1;
		gridbag.setConstraints(m_style_Combobox, c);
		ecPanel.add(m_style_Combobox);
		
		JLabel e_size_Label = new JLabel("size");
		m_size_Combobox = new JComboBox();
		for(int i = 8; i<=100; i++ ) m_size_Combobox.addItem(new Integer(i));

		c.gridx		= 0;	c.gridy		= 2;
		gridbag.setConstraints(e_size_Label, c);
		ecPanel.add(e_size_Label);
		
		c.gridx		= 1;	c.gridy		= 2;
		gridbag.setConstraints(m_size_Combobox, c);
		ecPanel.add(m_size_Combobox);
		
		//Create Printer Customize Panel		
		JPanel pcPanel = new JPanel();
		pcPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Printer Font" ));
		
		gridbag = new GridBagLayout();
		c = new GridBagConstraints();
		pcPanel.setLayout( gridbag );
		
		JLabel p_select_Label = new JLabel("Select:");
			
		JLabel p_font_Label = new JLabel("font");
		m_printer_style_Combobox = new JComboBox( s );
		
		c.gridwidth	= 2;
		c.weightx	= 0.1;	c.weighty	= 0.1;
		c.gridx		= 0;	c.gridy		= 0;
		gridbag.setConstraints(p_select_Label, c);
		pcPanel.add(p_select_Label);
		
		c.gridwidth	= 1;
		c.gridx		= 0;	c.gridy		= 1;
		gridbag.setConstraints(p_font_Label, c);
		pcPanel.add(p_font_Label);
			
		c.gridx		= 1;	c.gridy		= 1;
		gridbag.setConstraints(m_printer_style_Combobox, c);
		pcPanel.add(m_printer_style_Combobox);
		
		JLabel p_size_Label = new JLabel("size");
		m_printer_size_Combobox = new JComboBox();
		for(int i = 8; i<=100; i++ ) m_printer_size_Combobox.addItem(new Integer(i));
		
		//Set current font and font-size
		setCurrentFont( );
				
		c.gridx		= 0;	c.gridy		= 2;
		gridbag.setConstraints(p_size_Label, c);
		pcPanel.add(p_size_Label);
		
		c.gridx		= 1;	c.gridy		= 2;
		gridbag.setConstraints(m_printer_size_Combobox, c);
		pcPanel.add(m_printer_size_Combobox);
		
		//Create Color Customize Panel		
		JPanel ccPanel = new JPanel();
		ccPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Editor Color" ));
		
		gridbag = new GridBagLayout();
		c = new GridBagConstraints();
		ccPanel.setLayout( gridbag );
		
		JLabel choose_Label = new JLabel("Choose color of:");
		
		JLabel comment_Label = new JLabel("comments");
		JLabel literal_Label = new JLabel("keywords");
		JLabel invalid_Label = new JLabel("invalids");
				
		m_comment_Button = new JButton("");
		m_comment_Button.addActionListener(this);
		m_literal_Button = new JButton("");
		m_literal_Button.addActionListener(this);
		m_invalid_Button = new JButton("");
		m_invalid_Button.addActionListener(this);
		
		setCurrentButtonColor( );
		
		c.weightx	= 0.1;	c.weighty	= 0.1;
		c.gridwidth	= 2;
		c.gridx		= 0;	c.gridy		= 0;
		gridbag.setConstraints(choose_Label, c);
		ccPanel.add(choose_Label);
		
		c.gridwidth	= 1;
		c.gridx		= 0;	c.gridy		= 1;
		gridbag.setConstraints(comment_Label, c);
		ccPanel.add(comment_Label);
				
		c.gridx		= 1;	c.gridy		= 1;
		gridbag.setConstraints(m_comment_Button, c);
		ccPanel.add(m_comment_Button);
		
		c.gridx		= 0;	c.gridy		= 2;
		gridbag.setConstraints(literal_Label, c);
		ccPanel.add(literal_Label);
				
		c.gridx		= 1;	c.gridy		= 2;
		gridbag.setConstraints(m_literal_Button, c);
		ccPanel.add(m_literal_Button);
		
		c.gridx		= 0;	c.gridy		= 3;
		gridbag.setConstraints(invalid_Label, c);
		ccPanel.add(invalid_Label);
				
		c.gridx		= 1;	c.gridy		= 3;
		gridbag.setConstraints(m_invalid_Button, c);
		ccPanel.add(m_invalid_Button);
					
		gridPanel.add( ecPanel );
		gridPanel.add( pcPanel );
		gridPanel.add( ccPanel );
		
		//Create Button panel
		JPanel buttonPanel = new JPanel();
		
		m_ok_Button = new JButton("OK");
		m_ok_Button.addActionListener(this);
		
		m_cancel_Button = new JButton("Cancel");
		m_cancel_Button.addActionListener(this);
		
		//buttonPanel.add(m_default_Button);
		buttonPanel.add(m_ok_Button);
		buttonPanel.add(m_cancel_Button);
		
		Container contentPane = getContentPane();
		contentPane.add(gridPanel, "Center" );
		contentPane.add(buttonPanel, "South" );
		
		setResizable(false);
		setVisible(true);
	}
	
	/**
	 *  Set current button color
	 */
	private void setCurrentButtonColor()
	{
		m_comment_Button.setBackground(new Color(m_PropertiesManager.getProperty(PropertiesManager.P_COMMENTCOLOR)));
		m_literal_Button.setBackground(new Color(m_PropertiesManager.getProperty(PropertiesManager.P_LITERALCOLOR)));
		m_invalid_Button.setBackground(new Color(m_PropertiesManager.getProperty(PropertiesManager.P_INVALIDCOLOR)));
	}
	
	/**
	 *  Display current font and size
	 */
	private void setCurrentFont()
	{
		int current = 0;
		for( int i = 0; i < m_style_Combobox.getItemCount( ); i++)
			if( ((String)m_style_Combobox.getItemAt(i)).equalsIgnoreCase(m_PropertiesManager.getEditorFont().getName() ) ) current = i;
		
		m_style_Combobox.setSelectedIndex( current );
		
		current = 0;
		for( int i = 0; i < m_size_Combobox.getItemCount( ); i++)
			if( ( (Integer) m_size_Combobox.getItemAt(i) ).intValue( ) == m_PropertiesManager.getProperty(PropertiesManager.P_EDITORFONTSIZE ) ) current = i;
		
		m_size_Combobox.setSelectedIndex( current );
		
		current = 0;
		for( int i = 0; i < m_printer_style_Combobox.getItemCount( ); i++)
			if( ((String)m_printer_style_Combobox.getItemAt(i)).equalsIgnoreCase(m_PropertiesManager.getPrinterFont().getName() ) ) current = i;
		
		m_printer_style_Combobox.setSelectedIndex( current );
		
		current = 0;
		for( int i = 0; i < m_printer_size_Combobox.getItemCount( ); i++)
			if( ( (Integer) m_printer_size_Combobox.getItemAt(i) ).intValue( ) == m_PropertiesManager.getProperty(PropertiesManager.P_PRINTERFONTSIZE ) ) current = i;
		
		m_printer_size_Combobox.setSelectedIndex( current );
	}		
	
	/**
	 *  Choose color
	 *
	 * @param: function button
	 */
	private int chooseColor( JButton o )
	{
		Color c = JColorChooser.showDialog( this, "Choose color", o.getBackground( ) );
		
		if( c != null )
		{
			o.setBackground( c );
			return c.getRGB( );
		}
		
		return -1;
	}

	/**
	 *  ActionListener
	 */
	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		if(source == m_ok_Button)
		{
			//editor
			int size = ((Integer)m_size_Combobox.getSelectedItem()).intValue();
			m_PropertiesManager.setEditorFont((String)m_style_Combobox.getSelectedItem(), size);
			
			//printer			
			size = ((Integer)m_printer_size_Combobox.getSelectedItem()).intValue();
			m_PropertiesManager.setPrinterFont((String)m_printer_style_Combobox.getSelectedItem(), size);
			
			setVisible(false);
		}
		else if(source == m_comment_Button )
		{
			int c = chooseColor( m_comment_Button );
			if( c != -1 ) m_PropertiesManager.setCommentColor( c );
		}
		else if(source == m_literal_Button )
		{
			int c = chooseColor( m_literal_Button );
			if( c != -1 ) m_PropertiesManager.setLiteralColor( c );
		}
		else if(source == m_invalid_Button )
		{
			int c = chooseColor( m_invalid_Button );
			if( c != -1 ) m_PropertiesManager.setInvalidColor( c );
		}
		else if(source == m_cancel_Button )
		{
			setVisible(false);		
		}
	}
}