package mic1.gui;
/*
 * @(#)MainFrame.java	1.0 01/Oct/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.10 20/Jan/2001
 * @Author  : PB
 *
 * Description : 
 * 
 * Main GUI MIC1 Developer class
 * 
***  

To Do:
	
Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
Pierre Bakker    01.10.2000 Created
Pierre Bakker	 14.11.2000 Stack Memory view adding changed
Pierre Bakker	 15.11.2000 Splash screen added in Thread
Richard Klos	 08.12.2000 Console added
Richard Klos	 13.12.2000 Stackview added
Pierre Bakker	 13.12.2000	New Editor Added
Richard Klos	 16.12.2000 changed StackView contructor call.
Arno Paanstra	 28.12.2000 Add CustomizeFrame and Simulator stores settings on exit. [close(),showCustomizeFrame()]
Arno Paanstra	 29.12.2000 Changed showCustomizeFrame() methode, updates now also the linenumbers after Customizing.
Pierre Bakker    13.01.2001 HTML-viewer added.
Pierre Bakker    13.01.2001 MainFrame adapted to work with Editor and HTMLViewer Objects.
Pierre Bakker	 14.01.2001 Linenumber setting added.
Richard Klos     16.01.2001 Removed all hex-strings as parameters to functions implemented for the Mic1OutputInterface.
W. de Witte      18.01.2001 Toevoeging van Look & Feel stuff
Richard Klos     19.01.2001 Inserted MemoryTable in Memory Monitor.
Pierre Bakker	 22.01.2001 ajusted the STACK_SPLITSIZE
Pierre Bakker	 22.01.2001 bug, Switching documents while running solved
Pierre Bakker	 22.01.2001 Cleaned up
Pierre Bakker	 23.01.2001 Update V1.2
Pierre Bakker	 27.06.2001 MIR seperation
Richard Klos     03.10.2001 Small modifications to use improved MemoryTable.
Tim Helsloot     11.03.2003 Small modifications to improve on-screen dimensions
*/

import mic1.util.*;
import mic1.ijvm.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;
import javax.swing.event.*;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.table.*;
import javax.swing.UIManager;
import javax.swing.SwingUtilities;

import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

import mic1.gui.editor.syntax.*;
import mic1.gui.editor.textarea.*;
import mic1.gui.editor.*;
import mic1.gui.*;


public class MainFrame extends JFrame implements ActionListener, Commands,ComponentListener, 
												 Labels, Mic1OutputInterface, MouseListener, 
												 FocusListener
{		
	private Mic1Interface 			m_Simulator;	
	private JTabbedPane	 			m_Desktop;
	private StackView 				m_StackView;
	private JPanel                  m_OutputView;
	private	JListOutput				m_OutputList;										//outputlist in output view
	private CpuView 				m_CPU;	
	private MemoryTable             m_memoryTable;
	private MyMenuBar				m_Menu;	
	private JFileChooser 			m_Chooser;
	private JSplitPane 				m_Dev_outputPane,
									m_DeveloperPane,
									m_Edit_memoryPane,
									m_MemoryPane;								
	private static boolean			m_bLineNrs			= false;
	private static int				m_iOpenFrameCount 	= 0;

	
	private static final Color		TAB_BGCOLOR			= Color.lightGray;	
	private static final int		OUTPUT_SPLITSIZE	= 200;							// horizontal split between output and developers pane
	private static final int		STACK_SPLITSIZE		= 180;							// vertical stack split in developers pane
	private static final int		MEMORY_SPLITSIZE	= 440;							// vertical cpu and memory monitor split
	private static final int		CPUMEM_SPLITSIZE 	= 312; 							// horizontal split between cpu an memory monitor
		
	private SplashWindow 			m_sw;
    private Image 					m_splashIm;	
	private ConsoleFrame m_console;
	
	private PropertiesManager 		m_PropertiesManager;

	protected static final String 	ICONPATH 			= "mic1/gui/images/";			// root path for icons
    // Possible Look & Feels
    public static final String 		mac_style      		= "com.sun.java.swing.plaf.mac.MacLookAndFeel";
    public static final String 		metal_style    		= "javax.swing.plaf.metal.MetalLookAndFeel";
    public static final String 		motif_style    		= "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
    public static final String 		win_style  	  		= "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
    // The current Look & Feel
    private String 					currentLookAndFeel 	= metal_style;	
			
	public MainFrame(Mic1Interface mic1)
	{
		super(TITLE.sLabel);	

		// Spash screen
		MediaTracker mt = new MediaTracker(this);
		m_splashIm = Toolkit.getDefaultToolkit().getImage(ICONPATH+"MIC1-Splash.jpg");
		mt.addImage(m_splashIm,0);
		try
		{	mt.waitForID(0);}
		catch(InterruptedException ie){}
		// starts new Splash screen thread
		m_sw = new SplashWindow(this,m_splashIm);
		
		m_PropertiesManager = new PropertiesManager();		
		m_Desktop = new JTabbedPane(JTabbedPane.BOTTOM);		
				
		// Simulator
		m_Simulator = mic1;
		setIconImage(Toolkit.getDefaultToolkit().getImage(ICONPATH+"logo.gif"));	
		
		// MainFrame placement
        int inset = 60;
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(0,0,screenSize.width, screenSize.height-inset);
				
       	// add Menubar
       	m_Menu = new MyMenuBar(this);       	
       	setJMenuBar(m_Menu); 		
				
        // add Toolbar        	
		getContentPane().add(m_Menu.createToolBar(), BorderLayout.NORTH);
		
		// construct gui
		getContentPane().add(createMainPane(), BorderLayout.CENTER);
		
		m_Menu.setEnabled(false, MyMenuBar.ALL_ITEMS);       
        // create Console
        m_console = new ConsoleFrame(this);        
				
        addWindowListener(new WindowAdapter()
        {	public void windowClosing(WindowEvent e)
        	{	close();	}
        });  
        
        m_Menu.executeCommand(JAVASTYLE.sLabel);
        setVisible(true); 
	}

//**************************CREATING THE GUI****************************************/    	
	/**
	 * split window development / output
	 */
	protected JSplitPane createMainPane()
	{
        m_Dev_outputPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT	, 
        							   createStackView()				, 
        							   createDeveloperPanel()			);        									  
        m_Dev_outputPane.setContinuousLayout(true);
        m_Dev_outputPane.setOneTouchExpandable(true);
        m_Dev_outputPane.setDividerLocation(STACK_SPLITSIZE);         
        m_Dev_outputPane.setDividerSize(10);
        m_Dev_outputPane.setResizeWeight(1.0);

        return m_Dev_outputPane;
		
	}

//*******************************STACK VIEW*****************************************/    
    /**
     * Creates stack view at the left of the screen
     */
    protected JPanel createStackView() 
    {
		m_StackView = new StackView(m_Simulator);
     	m_StackView.addMouseListener(this);
		m_StackView.setBorder(BorderFactory.createTitledBorder("Stack monitor"));

    	return m_StackView;
    }

	
	/**
     * Splits screen, bottom output view, top developers view
     */ 
    protected JSplitPane createDeveloperPanel()
    {
        m_DeveloperPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, 
        							   createEditview_MemoryPanel()			, 
        							   createOutputview()	);
		m_DeveloperPane.setContinuousLayout(true);
		m_DeveloperPane.setOneTouchExpandable(true);
		m_DeveloperPane.setDividerLocation(getHeight() - OUTPUT_SPLITSIZE);
		m_DeveloperPane.setDividerSize(10);
		m_DeveloperPane.setResizeWeight(1.0);

        return m_DeveloperPane;
	}

//******************************OUTPUT VIEW*****************************************/    
	/**
	 * Creates output view at bottom of screen
	 */
	protected JPanel createOutputview()
	{
    	m_OutputView = new JPanel();
    	m_OutputView.setLayout(new BorderLayout());

		m_OutputList = new JListOutput();
		JScrollPane scrollPane = new JScrollPane(m_OutputList);
     	m_OutputList.addMouseListener(this);
     	m_OutputView.add("Center", scrollPane);
 		
 		m_OutputView.setBorder(BorderFactory.createTitledBorder("Process Output"));
    	return m_OutputView;
 	}
   	
/******************************DESKTOP VIEW****************************************/    	
	/**
	 * Splits screen, m_Desktop view left, cpu and memory views right
	 */
	protected JSplitPane createEditview_MemoryPanel()
	{
		m_Edit_memoryPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT	, 
        								 createDesktopView()			, 
        								 createMemoryViews()			);
		m_Edit_memoryPane.setContinuousLayout(true);
		m_Edit_memoryPane.setOneTouchExpandable(true);
		m_Edit_memoryPane.setDividerLocation(getWidth() - MEMORY_SPLITSIZE);
		m_Edit_memoryPane.setDividerSize(10);
		m_Edit_memoryPane.setResizeWeight(1.0);
        
		return m_Edit_memoryPane;
	}
	
	/**
     * Creates tabbed m_Desktop view in center of the screen 
     */
	protected JTabbedPane createDesktopView()
	{
		m_Desktop.addMouseListener(this);        
		m_Desktop.addComponentListener(this);
					
		return m_Desktop;
	}

/***************************CPU AND MEMORY VIEW************************************/
	/**
     * Splits screen, top CPU view, bottom memory
     */
    protected JSplitPane createMemoryViews()
    {
		m_MemoryPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT	, 
        							createCPUView()			, 
        							creatememoryTable()			);
		m_MemoryPane.setContinuousLayout(true);
		m_MemoryPane.setOneTouchExpandable(true);
		m_MemoryPane.setDividerLocation(CPUMEM_SPLITSIZE);
		m_MemoryPane.setDividerSize(10);
		m_MemoryPane.setResizeWeight(0.0);
        
		return m_MemoryPane;
    }
	/**
     * Creates CPU view at the right top of the screen
     */
	protected JScrollPane createCPUView()
    {    	
  		m_CPU = new CpuView(this);
  		m_CPU.setEnabled(false, MyMenuBar.ALL_ITEMS);
   		JScrollPane scrollPane = new JScrollPane(m_CPU);   	
	   	return scrollPane;
    }
	/**
     * Creates memory and watch view at the right bottom of the screen    
     */
	protected JPanel creatememoryTable()
	{
		m_memoryTable = new MemoryTable(m_Simulator, 0, 262143, false);
 		m_memoryTable.setBorder(BorderFactory.createTitledBorder("Memory monitor"));
 		m_memoryTable.setMinimumSize(new Dimension(0,0));
		return m_memoryTable;
    }

/*************************CREATE INTERNALFRAMES*************************************/    
	/**
     * Creates new empty Editor in TabbedPane
     */ 
	protected Editor createInternalFrame() 
    {
		
		ImageIcon doc = new ImageIcon(ICONPATH+"doc.gif");

		Editor editor;
        editor = new Editor(this, "Document #" + ++m_iOpenFrameCount);        
        editor.setFont(new Font("Courier", 0, 20));
        editor.addComponentListener(this);
        
		m_Desktop.addTab(editor.getTitle(),doc,  editor);		
        m_Desktop.setBackgroundAt(m_Desktop.getTabCount()-1, TAB_BGCOLOR);        
        makeSelected();		
        m_Desktop.addFocusListener(this);        
		return editor;
    }

	/**
     * Creates new Editor in TabbedPane
     *
     * @param : File to be opend
     */ 
    protected Editor createInternalFrame(File file) 
    {
		ImageIcon doc = new ImageIcon(ICONPATH+"doc.gif");
		
		Editor editor;
    	editor = new Editor(this, file);
    	editor.addComponentListener(this);
    	
		m_Desktop.addTab(editor.getTitle(),doc,  editor);
        m_Desktop.setBackgroundAt(m_Desktop.getTabCount()-1, TAB_BGCOLOR);
        makeSelected();               
        m_Desktop.addFocusListener(this);    
        return editor;
    }

	/**
     * Creates Help contents in TabbedPane
     */ 
    protected  HTMLViewer createInternalHelpFrame() 
    {
		ImageIcon doc = new ImageIcon(ICONPATH+"help.gif");
		
		HTMLViewer help;
		
		try
		{
			help = new HTMLViewer("file", null, "help//", "default.html");
			help.addComponentListener(this);
			m_Desktop.addTab(CONTENTS.sLabel, doc,  help);
		}
		catch(java.net.MalformedURLException e)
		{	System.out.println("Error: "+e);
			return null;
		}
    	
        m_Desktop.setBackgroundAt(m_Desktop.getTabCount()-1, TAB_BGCOLOR);
        
        makeSelected();       
        m_Desktop.addFocusListener(this);    
		
        return help;
    }
	
    /** 
     * Creates the about MIC1 Developer helpbox
     */												
 	public void createAboutDialog()
	{
		JDialog about = new JDialog(this, TITLE_ABOUT.sLabel+" "+TITLE.sLabel, true);
		  about.setSize(300,240);
		  about.setLocation(getWidth()/2-150, getHeight()/2-150);
		  about.getContentPane().setLayout(new BorderLayout());
		  
		  JPanel p0 = new JPanel();		  	
		  	ImageIcon logo = new ImageIcon(ICONPATH+"logo.gif");
		  	JLabel title = new JLabel(TITLE.sLabel + " " + VERSIONNR.sLabel, logo,JLabel.CENTER);
		  	p0.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED, Color.white, Color.black));
		  	p0.add(title);
		  
		  JPanel p1 = new JPanel()
		  {
		  	public void paint(Graphics g)
		  	{		 		
		  		g.setColor(Color.black);
		  		g.drawString("This product is freeware. It was build for",10,20);
		  		g.drawString("educational purposes.",10,35);		  	
		  		
		  		g.drawString("Developers:", 10, 65);
		  		g.drawString("P.G.J. Bakker", 100, 65);
		  		g.drawString("A. Chu", 100, 80);
		  		g.drawString("R. Klos", 100, 95);
		  		g.drawString("A.W.H. Paanstra", 100, 110);
		  		g.drawString("W.D. de Witte", 100, 125);
		  		g.drawString(COPYRIGHT.sLabel,30 ,155);
		  	}
		  };		  
		  
		  about.getContentPane().add("North",p0);
		  about.getContentPane().add("Center",p1);
		  about.setVisible(true);		
	}
	
    /**
     * Terminates the program
     */
	public void close()
	{
		closeAllDocuments();
		m_PropertiesManager.storeProperties();		
		System.exit(0);	
	}

    /**
     * Stores the current L&F, and calls updateLookAndFeel, below
     */
    public void setLookAndFeel(String laf) 
    {
    	if(currentLookAndFeel != laf) 
    	{
    		currentLookAndFeel = laf;
    		updateLookAndFeel();
    	}
    }

    /**
     * Sets the current L&F on each module
     */
    public void updateLookAndFeel() 
    {
    	try 
    	{
    		UIManager.setLookAndFeel(currentLookAndFeel);
    		SwingUtilities.updateComponentTreeUI(this);
    	} 
    	catch (Exception ex) 
    	{
    		System.out.println("Failed loading L&F: " + currentLookAndFeel);
    		System.out.println(ex);
    	}
    }

	/** 
     * Returns the MIC1Interface
     */
	public Mic1Interface getSimulator()
	{	return m_Simulator;	}
		
	/** 
     * Returns the PropertiesManager object
     */
	public PropertiesManager getPropertiesManager()
	{	return m_PropertiesManager;	}

    /** 
     * Sets the line numbers visible for all InternalDocuments
     */												
	public void setEditorLineNrs(boolean bState)
	{
		JComponent alldocs[] = getAllDocuments();
		
		for(int i=0; i<alldocs.length; i++)
		{
			if(alldocs[i] instanceof Editor)
				((Editor)m_Desktop.getComponentAt(i)).setLineNr(bState);
		}
	}

    /** 
     * Shows the console.
     */												
	public void showConsole()
	{
		m_console.setVisible(true);
	}
	

	/** 
     * Shows the console.
     */												
	public void showCustomizeFrame()
	{
		new CustomizeFrame( this, getPropertiesManager() );
		for(int i=0; i<m_Desktop.getTabCount(); i++)
		{
			JComponent tmp = (JComponent)m_Desktop.getComponentAt(i);
			if(tmp instanceof Editor)
			{
				((Editor)tmp).updateEditorFont();
				((Editor)tmp).setDocumentStyle();
				((Editor)tmp).repaint();
			}
		}	
	}
	
	/** 
     * Returns the component to witch the output is redirected
     */												
    public Output getOutput()
    {
    	return m_OutputList;
    }
    
    /** 
     * Returns the Component in the selected tab
     */												
    public JComponent getSelectedDocument()
    {
    	return (JComponent)m_Desktop.getSelectedComponent();
    }
    
    /**
     * Returns an array of all Components of the TabbedPane
     */
    public JComponent[] getAllDocuments()
    {
    	int nDocs = m_Desktop.getTabCount();
    	if(nDocs < 0)
    		return null;
    		
    	JComponent[] AllDocs = new JComponent[m_Desktop.getTabCount()];
			
		for(int i=0; i<nDocs;i++)				
			AllDocs[i] = (JComponent)m_Desktop.getComponentAt(i);
    	
    	return AllDocs;
    }
    
    /** 
     * Returns true if currently viewed Editor is closed succesfully
     */												
    public void closeSelectedDocument()
    {	
    	JComponent jcomp = getSelectedDocument();
    	
		if(jcomp instanceof Editor)
		{
			Editor tmp = (Editor)jcomp;

			if(tmp.isDirty())
			{
				int n = JOptionPane.showConfirmDialog(tmp,
												  MESS_SAVEFILE+tmp.getTitle(), 
												  TITLE_MODIFIED.sLabel,
												  JOptionPane.YES_NO_CANCEL_OPTION	);
				if (n == JOptionPane.YES_OPTION)
				{
					fileChooser(MyMenuBar.SAVE_ITEM);	
					m_Desktop.remove(tmp);
				}
				if (n == JOptionPane.NO_OPTION)
					m_Desktop.remove(tmp);
				if (n == JOptionPane.CANCEL_OPTION);	//do nothing				
			}
			else
				m_Desktop.remove(tmp);	
		}
		if(jcomp instanceof HTMLViewer)
		{
			HTMLViewer tmp = (HTMLViewer)jcomp;
			m_Desktop.remove(tmp);
		}	


     }
    
    /** 
     * Closes all components in the tabbedpane m_Desktop
     */												
    public void closeAllDocuments()
    {
		JComponent[] allDocs = getAllDocuments();

		if(allDocs != null)
		{			
			for(int i=0; i < allDocs.length; i++)
			{	m_Desktop.setSelectedComponent(allDocs[i]);
				closeSelectedDocument();
			}
		}
    }
    
    /** 
     * Returns the JTabbedPane component m_Desktop
     */												
	public JTabbedPane getDesktop()
    {
    	return m_Desktop;
    }


	/**
	 * Starts the Simulater when  the program loaded is valid
	 * and when the simulator is not already running
	 *
	 * @param : if true simulater wil execute in step mode
	 *			else it wil run the entire program continously
	 */
	public void startSimulator(String sFileName, boolean bStep)
	{
		try
		{	
			if(!m_Simulator.isRunning())	// cant happen
			{
				if(!m_Simulator.validProgram())			// Is this stil needed?
					m_Simulator.loadProgram(sFileName); // Is this stil needed?
				if(m_Simulator.validProgram())
				{
					if(bStep)
						m_Simulator.step();
					else
						m_Simulator.run();
				}
				else
				{
					System.out.println("Not A Valid Program");
				}
			}			
		}
		catch(java.io.IOException ioe)
		{
			System.err.println(ioe);
		}
		catch(Exception me)
		{
			System.err.println(me);
		}
	}
	
	
	public void runToBreakpoint(String sFileName)
	{
		try
		{	
			if(!m_Simulator.isRunning())	// cant happen
			{
				if(!m_Simulator.validProgram())			
					m_Simulator.loadProgram(sFileName); 
				if(m_Simulator.validProgram())
				{
					m_Simulator.runToBreakPoint();
				}
				else
				{
					System.out.println("Not A Valid Program");
				}
			}			
		}
		catch(java.io.IOException ioe)
		{
			System.err.println(ioe);
		}
		catch(Exception me)
		{
			System.err.println(me);
		}
	}	
	/**
	 * Stops the Simulater when running
	 */
	public void stopSimulator()
	{
		if(m_Simulator.isRunning())
		{
			m_Simulator.stop();						
		}
	}
	
	/**
	 * resets the simulator to start condition
	 */
	protected void resetSimulator()
	{
		try
		{			
			if(!m_Simulator.isRunning())
				m_Simulator.unloadProgram();
		}
		catch(Exception e)
		{	
			m_OutputList.println("Unable to reset");
			System.out.println("Error: "+e);
		}			
	}
	
	/**
	 * Send a character coming from the console.
	 */
	public void sendCharacter(char c)
	{
		m_Simulator.sendChar(c);
	}


/*************************Mic1OutputInterface**************************************/
    /** 
     * Invoked by the machine just before starting the thread.
     *
     */	
	public void OnRun()
	{		
		if(!m_Simulator.isRunning())
		{
			m_Menu.setEnabled(false, MyMenuBar.ONRUN_ITEMS);
			m_CPU.setEnabled(false, MyMenuBar.ONRUN_ITEMS);
		}
	}
	
    /** 
     * The machine stopped at a breakpoint position. This function
     * is called after OnStop();
     * 
     * @param : the linenumber of the breakpoint
     */		
	public void OnReachedBreakPoint(int nLineNumber)
	{
		((Editor)getSelectedDocument()).setHightLight(nLineNumber);
	}

    /** 
     * The machine thread has been stopped 
     * 
     * @param : the linenumber of the current instruction
     * @param : the current cycle (step count)
     */			
	public void OnStop(int nLineNumber, int nCycleCount)
	{
		m_Menu.setEnabled(true, MyMenuBar.ALL_ITEMS);
		m_CPU.setEnabled(true, MyMenuBar.ALL_ITEMS);
	}
	
    /** 
     * The machine is going to execute the current cycle
     * 
     * @param : the linenumber of the current instruction.
     * @param : the current cycle (step count)
     */				
	public void OnStep(int nLineNumber, int nCycleCount)
	{
		if(!m_Simulator.isRunning())
		{
			if(nLineNumber != -1) //highlight the steped into line
			{
				Editor tmp = (Editor)getSelectedDocument();
				if(tmp != null)
					tmp.setHightLight(nLineNumber);
			}			
		}		
		m_StackView.updateView();
		m_memoryTable.updateView();
	}

    /** 
     * Invoked by the machine when it sends a character
     * to the console.
     *
     * @param : a character.
     */											
	public void OnCharOut(char charOut)
	{
		m_console.writeChar(charOut);
	}
	
    /** 
     * The value of the MAR has been changed
     * 
     * @param : value of the register
     */		
	public void OnChangedMAR(int nValue)
	{	
		m_CPU.setRegister(nValue, Registers.MAR);	
	}
	
    /** 
     * The value of the MDR has been changed
     * 
     * @param : value of the register
     */			
	public void OnChangedMDR(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.MDR);	
	}
	
    /** 
     * The value of the PC has been changed
     * 
     * @param : value of the register
     */				
	public void OnChangedPC(int nValue)
	{	
		m_CPU.setRegister(nValue, Registers.PC);	
	}
	
    /** 
     * The value of the MBR has been changed
     * 
     * @param : value of the register
     */					
	public void OnChangedMBR(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.MBR);	
	}
	
    /** 
     * The value of the LV has been changed
     * 
     * @param : value of the register
     */						
	public void OnChangedLV(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.LV);
		m_StackView.setLV(nValue);	
	}
	
    /** 
     * The value of the CPP has been changed
     * 
     * @param : value of the register
     */							
	public void OnChangedCPP(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.CPP);	
	}
	
    /** 
     * The value of the TOS has been changed
     * 
     * @param : value of the register
     */								
	public void OnChangedTOS(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.TOS);
	}
	
    /** 
     * The value of the OPC has been changed
     * 
     * @param : value of the register
     */									
	public void OnChangedOPC(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.OPC);
	}
	
    /** 
     * The value of the H has been changed
     * 
     * @param : value of the register
     */										
	public void OnChangedH(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.H);
	}
	
    /** 
     * The value of the MI has been changed
     * 
     * @param : string representation of current micro-instruction
     * @param : string representation of next micro-instruction
     */										
	public void OnChangedMIR(String sValue, String sNextValue)
	{
		m_CPU.setMIRRegister(sValue, sNextValue);
	}
	
    /** 
     * The value of the MPC has been changed
     * 
     * @param : value of the register
     */											
	public void OnChangedMPC(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.MPC);
	}
	
    /** 
     * The stack pointer
     * 
     * @param : value of the stackpointer.
     */												
	public void OnChangedSP(int nValue)
	{
		m_CPU.setRegister(nValue, Registers.SP);
		m_StackView.setSP(nValue);	
	}	


    /** 
     * The values of the ALU has been changed
     * 
     * @param : inputline a
     * @param : inputline b
     * @param : outputline
     */												
	public void OnChangedALU(boolean n, boolean z, int out)
	{
		m_CPU.setNFlag(n);
		m_CPU.setZFlag(z);
	}

    /** 
     * The standard output of the machine
     * 
     * @param : output text
     */											
	public void OnChangedStandardOut(String sText)
	{	
		getToolkit().beep();
		m_OutputList.clear();	
		m_OutputList.println(sText);
	}
	
    /** 
     * The debug output of the machine
     * 
     * @param : output text
     */											
	public void OnChangedDebugOut(String sText)
	{		
		if(sText.length() > 0)
			m_OutputList.print(sText);
	}

    /** 
     * Writes a character to the console.
     * 
     * @param : a character.
     */											
	public void writeCharToConsole(char charOut)
	{
		m_console.writeChar(charOut);
	}


    /** 
     * Addes the selected file name to the program title and Tab
     *    
     * @param : Currently view Editor title
     */
    private void extendTitle(String subtitle)
    {
    	if(subtitle != null)
    		setTitle(TITLE +" - [ "+ subtitle+" ]");
    	
    	if(m_Desktop != null)
		{
			int activeTab = m_Desktop.getSelectedIndex();   				
    		if(activeTab != -1)
    			m_Desktop.setTitleAt(activeTab, subtitle);
    	}    	
    }
    
    /** 
     * Selectes the opend last Editor
     */												
    private void makeSelected()
    {
    	m_Desktop.setSelectedIndex(m_Desktop.getTabCount()-1);
    }

    /** 
     * creates the file chooser object          
     */												
	private void createFileChooser()
	{	
		m_Chooser = new JFileChooser();
		AssemblerFileFilter filter = new AssemblerFileFilter();
		filter.addExtension("jas");		
		filter.addExtension("mal");
		filter.setDescription("MIC1 assembler files");
		m_Chooser.setFileFilter(filter);		
	}
		
    /** 
     * Compares the selected file from the filechooser and the file 
     * currently opend. If the same, dialog pops up. 
     *
     * @param : selected file from file chooser dialog
     */												
	private boolean isAllreadyOpen(File file)
	{
		if(m_Desktop.getTabCount() == 0)	//no internal frames
			return false;
				
		Editor tmp;
		String openFile;
		String selectedFile = file.getPath();
		
		for(int i=0; i<m_Desktop.getTabCount();i++)
		{			
			if(m_Desktop.getComponentAt(i) instanceof Editor)
			{
				tmp = (Editor)m_Desktop.getComponentAt(i);
				if(!tmp.isNew())	//to avoid nullpointerexception	
				{
					openFile = tmp.getPath();					
					if(openFile.equals(selectedFile))
					{
						m_Desktop.setSelectedIndex(i); // focus to opend file
						return true;
					}
				}
			}
		}
		return false;
	}	

    /** 
     * Calles the save or open file dialog        
     * 
     * @param : action to be preformed: open, save, save as, save all
     */												
	protected void fileChooser(int action)
	{
		if(m_Chooser == null)
			createFileChooser();
				 	
		// Open file 
		if(action == MyMenuBar.OPEN_ITEM)
		{
			m_Chooser.setMultiSelectionEnabled(true);
			m_Chooser.setDialogTitle(OPEN.sLabel);
			//active
			int returnVal = m_Chooser.showOpenDialog(this);
			if(returnVal == JFileChooser.APPROVE_OPTION)
			{
				File[] selectedFiles = m_Chooser.getSelectedFiles();
			
				for(int i=0; i<selectedFiles.length; i++)				
				{				
					if(isAllreadyOpen(selectedFiles[i]))
					{
						JOptionPane.showMessageDialog(this,
													  MESS_ISOPEN.sLabel, TITLE_ISOPEN.sLabel,
													  JOptionPane.ERROR_MESSAGE);
					}
					else
						createInternalFrame(selectedFiles[i]);
				}
				try
				{	m_Chooser.setCurrentDirectory(selectedFiles[0]);	}
				catch(ArrayIndexOutOfBoundsException aoobe)
				{}
					
				makeSelected();
			}
		}		
		// save file
		else if(action == MyMenuBar.SAVE_ITEM)
		{
			Editor tmp = (Editor)getSelectedDocument();
			if(tmp != null)
			{
				if(tmp.isNew())
					fileChooser(MyMenuBar.SAVEAS_ITEM);
				else
					tmp.saveYourSelf();
			}											
		}
		// save file as
		else if(action == MyMenuBar.SAVEAS_ITEM)
		{
			m_Chooser.setMultiSelectionEnabled(false);	
			m_Chooser.setDialogTitle(SAVEAS.sLabel);
		    int returnVal = m_Chooser.showSaveDialog(this);
		    		    
		    if(returnVal == JFileChooser.APPROVE_OPTION)
		    {		    	
		    	Editor tmp = (Editor)getSelectedDocument();
		    	File file = m_Chooser.getSelectedFile();	    		    	
		    	
		    	if(file.exists())
		    	{		    		
		    		int n = JOptionPane.showConfirmDialog(tmp,
														  MESS_OVERWRITE.sLabel+file.getName(),
														  TITLE_EXSITS.sLabel,
														  JOptionPane.YES_NO_OPTION	);
                    if (n == JOptionPane.YES_OPTION)
                    {
                    	
                    	if(isAllreadyOpen(file))
                    	{
							JOptionPane.showMessageDialog(tmp,
														  MESS_ISOPEN.sLabel, TITLE_ISOPEN.sLabel,
														  JOptionPane.ERROR_MESSAGE);
						}
        	            else
            	        {
            	        	tmp.saveYourSelf(file);				
							extendTitle(file.getName());							
							m_Chooser.setCurrentDirectory(file);
						}
                	}
                }
		    	else 
		    	{	    		
		    		tmp.saveYourSelf(file);
		    		extendTitle(file.getName());		    	
		    	}	    	
			}				
		}
		// sava all files
		else if(action == MyMenuBar.SAVEALL_ITEM)
		{
			Editor[] allFrames = new Editor[m_Desktop.getTabCount()];			
			for(int i=0; i<allFrames.length;i++)				
				 allFrames[i] = (Editor)m_Desktop.getComponentAt(i);
			
			for(int i=0; i< allFrames.length; i++)
			{
				Editor tmp = allFrames[i];				
				if(!tmp.isNew())
					tmp.saveYourSelf();
				else
				{
					int n = JOptionPane.showConfirmDialog(tmp,
														  MESS_ISNEW+tmp.getTitle(), 
														  TITLE_NEWFILE.sLabel,
														  JOptionPane.YES_NO_OPTION	);
                    if (n == JOptionPane.YES_OPTION)
                    {                		
                		m_Desktop.setSelectedComponent(tmp);                		
                		fileChooser(MyMenuBar.SAVEAS_ITEM);
                	}                	
				}
			}			
		}
	}    

/******************************EVENT HANDLERS**************************************/	

	/**
	 * MouseListener
	 */
	public void mouseReleased(MouseEvent ev)		{}
	public void mouseEntered(MouseEvent ev)			{}
	public void mouseExited(MouseEvent ev) 			{}
	public void mousePressed(MouseEvent ev)
	{				
		if (ev.getClickCount() == 2 && ev.getModifiers() / MouseEvent.BUTTON1_MASK == 1) // left mouse button
		{
			JComponent jcomp = (JComponent)getSelectedDocument();	
			if(m_OutputList != null && jcomp instanceof Editor)
			{
				int nIndex = m_OutputList.locationToIndex(ev.getPoint());
				int nLine = m_OutputList.getLineNrAt(nIndex);				
				
				Editor tmp =(Editor)jcomp;
				if(tmp != null && nLine > 0)
					tmp.setHightLight(nLine);			
			}
        }
	}	
	public void mouseClicked(MouseEvent ev)			{}

	/**
	 * ActionListener
	 */
	public void actionPerformed(ActionEvent e)
	{		
		// button action
		if(e.getSource() instanceof JButton)
			m_Menu.executeCommand( ((JButton)e.getSource()).getText());
	}
	
	/**
	 * ComponentListener
	 */
	public void componentResized(ComponentEvent e)	
	{
	//	m_StackView.reshape();
	}
	public void componentMoved(ComponentEvent e)	{}
	public void componentShown(ComponentEvent e)
	{			
		if(e.getComponent() instanceof Editor)
		{
			Editor tmp = (Editor)e.getComponent();
			tmp.setFocus();		//transfer Focus to the Editor Component
			
			extendTitle(tmp.getTitle());
			m_Menu.setEnabled(true, MyMenuBar.ALL_ITEMS);
			m_CPU.setEnabled(true, MyMenuBar.ALL_ITEMS);
			m_Menu.setDocumentCloseButton(true);
			tmp.setLineNr(m_PropertiesManager.isLineNrEnabled());

		}
		if(e.getComponent() instanceof HTMLViewer)
		{
			HTMLViewer tmp = (HTMLViewer)e.getComponent();
			tmp.setFocus();
			
			extendTitle(CONTENTS.sLabel);
			m_Menu.setEnabled(false, MyMenuBar.ALL_ITEMS);
			m_CPU.setEnabled(false, MyMenuBar.ALL_ITEMS);
			m_Menu.setDocumentCloseButton(true);

		}		
	}
	public void componentHidden(ComponentEvent e)	
	{			
		if(m_Simulator.isRunning())
			stopSimulator();
		if(e.getComponent() instanceof Editor)
		{
			try
			{	
				if(m_Simulator.validProgram())
					m_Simulator.unloadProgram();
			}
			catch(Exception ex)
			{	System.out.println("Unable to unload program.");	}
	
			if(m_Desktop.getTabCount() == 0)	//if no Document is opend
			{		
				m_Menu.setEnabled(false, MyMenuBar.ALL_ITEMS);
				m_CPU.setEnabled(false, MyMenuBar.ALL_ITEMS);
				m_Menu.setDocumentCloseButton(false);
				setTitle(TITLE.sLabel);	//reset program title		
			}
		}
	}
	
	/**
	 * focusListener
	 */
	public void focusGained(FocusEvent e)
	{		
		if(getSelectedDocument() instanceof Editor)
		{	
			Editor tmp = (Editor)getSelectedDocument();
	
			// check if InternalDocument is changed from outside the editor
			if(tmp.isUpdated())
			{
				int n = JOptionPane.showConfirmDialog(tmp,
													  MESS_ISMODIFIED+tmp.getTitle(), 
													  TITLE_MODIFIED.sLabel,
													  JOptionPane.YES_NO_OPTION	);
				if (n == JOptionPane.YES_OPTION)
					tmp.reopenFile();				
				else
					tmp.saveYourSelf();
			}
		}
	}
	public void focusLost(FocusEvent e)	{}
}




class SplashWindow extends Window implements Runnable, Labels
{
	private Thread thr;
    private	Image 							m_SplashIm;
   
    SplashWindow(Frame parent, Image splashIm)
    {
		super(parent);
		m_SplashIm = splashIm;
		setSize(500,300);
		
		// Center the Splash
		Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
		Rectangle winDim = getBounds();
		setLocation((screenDim.width - winDim.width) / 2,(screenDim.height - winDim.height) / 2);
		setVisible(true);
		
		// continu running parent
		thr = new Thread(this);
		thr.start();
	}
    
    public void run()
    {
    	try
		{	thr.sleep(6000);	}
		catch(InterruptedException ie){}				
		dispose();
		//thread dies
    }

    public void paint(Graphics g)
    {
    	if (m_SplashIm != null)
    	{    		
    		g.drawImage(m_SplashIm,0,0,this);
    		
    		g.setFont(VERSION.getFont());
    		g.drawString(TITLE.sLabel+" "+VERSION.sLabel, 325, 15);
    		g.drawString(VERSIONNR.sLabel+" ", 325+147, 15);
    		
    		g.setFont(COPYRIGHT.getFont());
    		g.drawString(COPYRIGHT.sLabel, 195, 286);
    		g.drawString(RIGHT.sLabel, 195, 296);
    		
    	}
    }
}
