package mic1.gui;
/*
 * @(#)StackView.java	1.01 16/Dec/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.20 19/Jan/2001
 * @Author  : PB
 *
 * Description : 
 * 
 * Creates the Stack view and stack related pointers
 * 
***  

To Do:



Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
Pierre Bakker    01.11.2000 Created
Pierre Bakker	 15.11.2000 Row count depending on panel height
Richard Klos     12.12.2000 Major reconstruction to enable scrolling.
Richard Klos     16.12.2000 StackView enabled to receive SP- and
                            LV-registers changes and ask MIC1Interface
                            for memory contents.
Richard Klos     18.01.2001 Major change. Separated StackView into
                            StackView and MemoryTable.
Pierre Bakker	 22.01.2001	Cleaned up.
Richard Klos     03.10.2001 Small modification to use improved MemoryTable.
Tim Helsloot     12.03.2003 Stackview starts at 0x10000 (Constant Pool)                            
*/

import mic1.ijvm.Mic1Interface;
import mic1.ijvm.MainMemory;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;

import javax.swing.table.DefaultTableModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JScrollBar;
import javax.swing.Scrollable;
import javax.swing.table.TableColumn;

public class StackView extends JPanel implements AdjustmentListener, ActionListener
{
	private MemoryTable				m_memoryTable;
	PointerPanel 					m_PointerPanel;
	Pointer                         m_pointerSP,
	                                m_pointerLV;
	
	private Mic1Interface			m_mic1;
	
	// Height of view in pixels.
	private	int m_viewHeight;
	
	// The values of the SP and LV registers.
	private int m_SP, m_LV;

	private JButton m_focusOnSP, m_focusOnLV;

	// The colors of the SP and LV pointers.
	private static final Color m_SPColor = Color.blue;
	private static final Color m_LVColor = Color.red;

	public StackView(Mic1Interface mi)
	{
		super();

		m_mic1       = mi;		

		m_memoryTable = new MemoryTable(m_mic1, 65536, MainMemory.MEM_MAX - 1, true);
		m_memoryTable.addAdjustmentListener(this);
		
		// pointers
		m_PointerPanel = new PointerPanel();
		m_pointerSP    = new Pointer(m_memoryTable.getRowHeight(), m_SPColor);
		m_pointerLV    = new Pointer(m_memoryTable.getRowHeight(), m_LVColor);
		m_PointerPanel.setPointers(m_pointerSP, m_pointerLV);

		// Pointer focus button panel.
		m_focusOnSP = new JButton();
		m_focusOnLV = new JButton();
		m_focusOnSP.setToolTipText("Center on SP");
		m_focusOnLV.setToolTipText("Center on LV");
		m_focusOnSP.setBackground(m_SPColor);
		m_focusOnLV.setBackground(m_LVColor);
		m_focusOnSP.addActionListener(this);
		m_focusOnLV.addActionListener(this);
		
		JPanel bp = new JPanel();
		bp.setLayout(new FlowLayout(5, 5, FlowLayout.CENTER));
		bp.add(m_focusOnSP);
		bp.add(m_focusOnLV);
				
		setLayout(new BorderLayout(0,0));		
		add(bp, BorderLayout.NORTH);
		add(m_PointerPanel, BorderLayout.WEST);
		add(m_memoryTable, BorderLayout.CENTER);		
	}

	
	/**
	 * Sets a new value to the Stack pointer	
	 *
	 * @param: table row number
	 */
	public void setSP(int sp)
	{
		m_SP = (sp);
	}

	/**
	 * Sets a new value to the Local Variable pointer	
	 *
	 * @param: table row number
	 */
	public void setLV(int lv)
	{
		m_LV = (lv);
	}

	/**
	 * Draws the new memory values and both the SP and LV arrows.
	 */
	public void updateView()
	{
		int top    = m_memoryTable.getTopAddress();
		int bottom = m_memoryTable.getBottomAddress();
		
		trySetPointer(top, bottom, m_SP, m_pointerSP);
		trySetPointer(top, bottom, m_LV, m_pointerLV);
				
		m_memoryTable.updateView();
		m_PointerPanel.repaint();
	}


	/**
	 * Attempts to place a pointer next to a specified memory address.
	 * If the address is currently not visible, the pointer wil be
	 * made invisible.
	 */
	private void trySetPointer(int top, int bottom, int wantedAddress, Pointer p)
	{
		if(top <= wantedAddress  &&  wantedAddress <= bottom)
		{
			p.placePointerOn(wantedAddress - top);
			p.setVisible(true);
		}
		else
			p.setVisible(false);
	}
	

	// AdjustmentListener
	public void adjustmentValueChanged(AdjustmentEvent e)
	{
		updateView();
	}
	

	// ActionListener
	public void actionPerformed(ActionEvent e)
	{
		Object obj = e.getSource();
		if(obj == m_focusOnSP)
			m_memoryTable.centerOn(m_SP);
		else if(obj == m_focusOnLV)
			m_memoryTable.centerOn(m_LV);
	}
}


class PointerPanel extends JPanel
{
	private static final Color		m_cBgColor = Color.lightGray;

	private Pointer 				m_SP;
	private Pointer 				m_LV;

	public PointerPanel()
	{
		setBackground(Color.green);
		setPreferredSize(new Dimension(10, getHeight()));
	}


	/**
	 * Makes the pointers visible
	 *
	 * @param: Stack Pointer
	 * @param: Local Variable Pointer
	 */
	public void setPointers(Pointer SP, Pointer LV)
	{
		m_SP = SP;
		m_LV = LV;
		m_SP.setOrder(1);
		m_LV.setOrder(2);
		m_SP.placePointerOn(0);
		m_LV.placePointerOn(0);
		m_SP.setVisible(true);
		m_LV.setVisible(true);
	}
			

	public void paint(Graphics g)
	{		
		g.setColor(m_cBgColor);
		g.fillRect(0,0, getWidth(), getHeight());				
		
		if(m_SP != null)
			m_SP.draw(g);
		if(m_LV != null)
			m_LV.draw(g);
	}
}

class Pointer
{
	private Color				m_cPointerColor;
	private Polygon				m_Triangle;
	private int					m_iPointerPlace	=	0,
								m_iOrder		=	1;
	private static final int	m_Npoints		=	3;
	private int[] 				m_Xpoints;
	private int[] 				m_Ypoints;
	
	private int					m_iColumnHeight;
	
	private boolean             m_visible;
	
	
	Pointer(int columnHeight)
	{	
		this(columnHeight, Color.black);
	}


	Pointer(int columnHeight, Color ponterColor)
	{	
		m_cPointerColor = Color.red;				
		m_iColumnHeight = columnHeight;
		m_cPointerColor = ponterColor;

		m_visible = true;
		
		m_Xpoints = new int[m_Npoints];
		m_Ypoints = new int[m_Npoints];
		
		setOrder(m_iOrder);
		placePointerOn(m_iPointerPlace);
	}


	/**
	 * makes the pointer visible or invisivle
	 *
	 * @param: boolean true = visible, false = invisible
	 */
	public void setVisible(boolean bState)
	{
		m_visible = bState;
	}
	
	
	/**
	 * returns true if the pointer is visible	 
	 */
	public boolean isVisible()
	{
		return m_visible;
	}


	/**
	 * Determans the drawing position for the pointer 
	 *
	 * @param: order number
	 */
	public void setOrder(int order)
	{
		m_iOrder = order;
		if(m_iOrder > 1)
		{
			m_Xpoints[0] = m_iOrder*3;
			m_Xpoints[1] = m_iOrder*5;
			m_Xpoints[2] = m_iOrder*3;
		}
		else
		{
			m_Xpoints[0] = m_iOrder;
			m_Xpoints[1] = m_iOrder*5;
			m_Xpoints[2] = m_iOrder;
		}				
	}
	

	/**
	 * Places the pointer
	 *
	 * @param : Index nr of the table 
	 */
	public void placePointerOn(int placement)
	{
		m_iPointerPlace = placement;
		
		if(m_Xpoints == null)
			setOrder(1);
		
		// Changed sign                                     -
		int iLineCenter = m_iColumnHeight * m_iPointerPlace + m_iColumnHeight / 2; 
		int iLineQuart = m_iColumnHeight / 3;
		m_Ypoints[0] = iLineCenter-iLineQuart;
		m_Ypoints[1] = iLineCenter;
		m_Ypoints[2] = iLineCenter+iLineQuart;
			
		m_Triangle = new Polygon(m_Xpoints, m_Ypoints, m_Npoints);
	}
	

	/**
	 * Paints the pointer
	 *
	 * @param : Graphics class
	 */
	public void draw(Graphics g)
	{
		if(m_visible)
		{
			g.setColor(m_cPointerColor);
			if(m_Triangle != null)
				g.fillPolygon(m_Triangle);
		}
	}
}
