/*
 * @(#)Editor.java 1.0 10/Dec/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.00 10/Dec/2000
 * @Author  : AC
 *
 * Description : 
 * 
 * Contains a JEditTextArea and takes care of the communication
 * between this object and any other. Extra Functionality is 
 * added for file operations
 *
***  

To Do:

Modification History:

Name			 Date	   Comment
---------------- ---------- ----------------------------------------
Andy Chu		 10.12.2000 Made compatible with MIC1 project
Pierre Bakker  	 13.12.2000 Ajusted to the Main GUI
Pierre Bakker	 15.12.2000 Breakpoint ajustment
Arno Paanstra	 28.12.2000 Add updateEditorFont() methode
Arno Paanstra	 14.01.2001 Change print methode: printing with a custom font
Andy Chu		 14.01.2001 Dynamic Font Adaptation added for gutter
Pierre Bakker	 14.01.2001 Linenumber setting changed, setLineNr(boolean bState) added
Pierre Bakker	 22.01.2001 Cleaned up

*/
package mic1.gui.editor;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.undo.*;
import java.net.URL;
import java.util.*;

import mic1.gui.editor.syntax.*;
import mic1.gui.editor.textarea.*;
import mic1.gui.*;

public class Editor extends JComponent
{
	private JEditTextArea			m_Textarea;
	private boolean					m_bIsNew   			= true;
	private long					m_lLastModified  	= 0;
	private File					m_fContext;
	private String	 				m_sTitle;
	private MainFrame				m_Parent;
	private int[]					m_Breakpoints;
	
	public Editor()
	{
		this(null, null, null);
	}

	public Editor(MainFrame parent)
	{
		this(parent, null, null);
	}
	public Editor(MainFrame parent,  String sTitle)
	{
		this(parent, sTitle, null);		 	
	}
	public Editor(MainFrame parent,  File importfile)
	{
		this(parent, null, importfile);		 	
	}
	public Editor(MainFrame parent,  String sTitle, File importfile)
	{
		super();
		setLayout(new BorderLayout());
		m_Parent = parent;
		m_sTitle = sTitle;
		m_fContext = importfile;
		m_Breakpoints = new int[10];
		
		m_Textarea=new JEditTextArea();
		m_Textarea.setBackground(Color.white);
		m_Textarea.getPainter().setFont(m_Parent.getPropertiesManager().getEditorFont());
		m_Textarea.setEditable(true);		
		m_Textarea.getGutter().setFont(m_Parent.getPropertiesManager().getEditorFont());
		
		if(m_fContext!=null)
			addContext(m_fContext);
		
		setDocumentStyle();		
		add(m_Textarea, BorderLayout.CENTER);
		m_Textarea.requestFocus();
	}
	
	/**
	 * Detects when the content of the textarea
	 * is changed from last time it was saved
	 */
	public boolean isDirty()
	{
		return m_Textarea.isDirty();
	}
	
	/**
	 * hightlight the line of the given line number
	 *
	 * @param : line number to be hightlighted
	 */
	public void setHightLight(int nLine)
	{
		if(nLine < 1)
			nLine=1;
		
		m_Textarea.setCaretPosition(m_Textarea.getSelectionStart(nLine-1));
		setFocus();
	}

	/**
	 * Creates a Print mic1.gui.Print object wich handels the Printing
	 */
	public void print()
	{
		Print print;		
		print = new Print((PlainDocument)m_Textarea.getDocument(), m_Parent.getPropertiesManager().getPrinterFont());	
	}

 	/**
	 * returns the title of this editor	  
	 */
	public String getTitle()
	{
		if(m_fContext == null)
			return m_sTitle;
		return m_fContext.getName();
	}

	public void cut()
	{	m_Textarea.cut();	 }
	public void copy()
	{	m_Textarea.copy();	}
	public void paste()
	{	m_Textarea.paste();	}
	public void selectAll()
	{	m_Textarea.selectAll();   }
	public void delete()
	{	m_Textarea.setSelectedText(""); }

 	/**
	 * Sets a new MicTokenMarker
	 */
	public void setDocumentStyle()
	{
		m_Textarea.setTokenMarker(new MicTokenMarker());
	}

 	/**
	 * read the text from the file and inserts it in the textarea
	 */
 	public void addContext(File file)
	{
		StringBuffer sb = new StringBuffer();
		try
		{
			BufferedReader in = new BufferedReader(new FileReader(file));
			String str = in.readLine();
			while (str != null)
			{
				sb.append(str);
				sb.append("\n");
				str = in.readLine();
			}
			in.close();
			m_Textarea.setText(sb.toString());
			m_Textarea.setCaretPosition(0);
			m_bIsNew = false;
			m_lLastModified = file.lastModified();
		}
		catch (Exception err)
		{
			System.err.println("Exception: "+err);
		}
	}

 	/**
	 * Returns true if textarea is not committed to a file
	 * else returns false
	 */
	public boolean isNew()
	{
  		return m_bIsNew;
	}	

 	/**
	 * Returns the path and file name of m_fContext 
	 */
 	public String getPath()
 	{
  		if(isNew())
   			return " ";
		return m_fContext.getPath();
	}

 	/**
	 * Overwrites file with text from textarea	 
	 *
	 * @param : destionation file
	 */
	public void saveYourSelf(File file)
	{
		m_fContext = file;		
		saveYourSelf();
	}

	/**
	 * Saves text from textarea to file
	 */
	public void saveYourSelf()
	{
		try
		{
			try
			{
				BufferedWriter out = new BufferedWriter(new FileWriter(m_fContext));
				String tmp = m_Textarea.getText();
				out.write(tmp);
				out.close();
			}
			catch(IOException ioe)
			{	System.out.println("Output failed. Error: " + ioe);	}
		}
		catch(NullPointerException npe)
		{	System.out.println("File error: " + npe); }
		
		m_bIsNew = false;
		m_lLastModified = m_fContext.lastModified();		
		m_Textarea.dirty = false;
		}

	/**
	 * Returns true if m_fContext.lastModified > m_lLastModified
	 */
	public boolean isUpdated()
	{
		if(isNew())
			return false;
		else if(m_fContext.lastModified() > m_lLastModified)
		{
			m_Textarea.requestFocus();
			return true;
		}
		return false;
	}
	
	/**
	 * Returns a long representation of the time the file has 
	 * last modified
	 */
	protected long lastModified()
	{
		if(m_fContext == null)
			return m_lLastModified;
		else
			return m_fContext.lastModified();
	}
		
	/**
	 * Requestes the focus
	 */
	public void setFocus()
	{
		m_Textarea.requestFocus();
	}
	
	/**
	 * Adds breakpoint on current line  
	 */
	public void addBreakpoint()
	{
		m_Textarea.addBreakpoint();
		try
		{
			(m_Parent.getSimulator()).setBreakPoint(m_Textarea.getCaretLine()+1);
		}
		catch(Exception e)
		{	System.out.println(e+" : unable to set breakpoint");	}
		setFocus();
	 }
		
	/**
	 * removes breakpoint on current line
	 */
	public void removeBreakpoint()
	{
	 	m_Textarea.removeBreakpoint();
	 	try
	 	{
	 		(m_Parent.getSimulator()).removeBreakPoint(m_Textarea.getCaretLine()+1);
	 	}
	 	catch(Exception e)
	 	{	System.out.println(e+" : unable to remove breakpoint");	}
	 	setFocus();
	}
	
	/**
	 * re-reads the context  
	 */
	public void reopenFile()
	{
	 	if(m_fContext != null)
	 		addContext(m_fContext);
	}
	
	/**
	 * Set the line numbers visible or invisible
	 *
	 * @param: boolean true = line numbers set, 
	 *				   false = line numbers not set
	 */
	public void setLineNr(boolean bState)
	{
	 	m_Textarea.setLineNrEnabled(bState);
	}
		
	/**
	 * Apply the Editors Font from the Propertiesmanager 
	 * to the textarea.
	 */
	public void updateEditorFont()
	{
		m_Textarea.getPainter().setFont(m_Parent.getPropertiesManager().getEditorFont());
	 	m_Textarea.getGutter().setFont(m_Parent.getPropertiesManager().getEditorFont());	//Andy
	}
}