/*
 * @(#)PropertiesManager.java	1.0 10/Dec/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.00 10/Dec/2000
 * @Author  : AC
 *
 * Description : 
 * 
 * contains Color properties, Font Properties and Bounds properties 
 * for the gui. 
 * 
***  

To Do:

Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
Andy Chu	     10.12.2000 Created
Pierre Bakker	 13.13.2000 Ajusted to the Main GUI
Arno Paanstra	 28.12.2000 Load and Store Properties
Arno Paanstra	 29.12.2000 Add color properties
Arno Paanstra	 31.12.2000 Add defaults
Arno Paanstra	 14.01.2001 Add printer font
Pierre Bakker	 14.01.2001 Linenumber Propertie added
Pierre Bakker	 22.01.2001 Cleaned up
*/

package mic1.gui.editor;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class PropertiesManager
{
	private	Properties					m_Properties;
	private File						m_propertiesFile;
	
	//Property keys
	public static String				P_EDITORFONT		= "EditorFont";
	public static String				P_EDITORFONTSIZE	= "EditorFontSize";
	public static String				P_PRINTERFONT		= "PrinterFont";
	public static String				P_PRINTERFONTSIZE	= "PrinterFontSize";
	public static String				P_COMMENTCOLOR		= "CommentColor";
	public static String				P_LITERALCOLOR		= "LiteralColor";
	public static String				P_INVALIDCOLOR		= "InvalidColor";
	public static String				P_LINENUMBER		= "Linenumbers";
	
	//Default values
	private final String				EDITORFONT			= "Monospaced";
	private final String				EDITORFONTSIZE		= "14";
	private final String				PRINTERFONT			= "Monospaced";
	private final String				PRINTERFONTSIZE		= "12";
	private final String				COMMENTCOLOR		= "39168";
	private final String				LITERALCOLOR		= "255";
	private final String				INVALIDCOLOR		= "255";
	private final String				LINENUMBER			= "false";
			
	private Font						m_MenuItemFont,				// menuitem font
										m_MenuFont;					// menu title font		
	public boolean						m_bLineNr;
	
    public static int					XPOS,YPOS,XSIZE,YSIZE;
    public static int[] 				MICCOLOR = {39168,39168,255,255,255,255,255,255,255,255};

    public PropertiesManager()
    {
    	//holds properties
        m_Properties = new Properties();
        
        //Load stored settings
        loadProperties("sim.conf");
        
        //Set defaults        
        if( m_Properties.getProperty(PropertiesManager.P_EDITORFONT) == null ) m_Properties.setProperty(PropertiesManager.P_EDITORFONT,EDITORFONT);
        if( m_Properties.getProperty(PropertiesManager.P_EDITORFONTSIZE) == null ) m_Properties.setProperty(PropertiesManager.P_EDITORFONTSIZE,EDITORFONTSIZE);
        if( m_Properties.getProperty(PropertiesManager.P_PRINTERFONT) == null ) m_Properties.setProperty(PropertiesManager.P_PRINTERFONT,PRINTERFONT);
        if( m_Properties.getProperty(PropertiesManager.P_PRINTERFONTSIZE) == null ) m_Properties.setProperty(PropertiesManager.P_PRINTERFONTSIZE,PRINTERFONTSIZE);if( m_Properties.getProperty(PropertiesManager.P_COMMENTCOLOR) == null ) m_Properties.setProperty(PropertiesManager.P_COMMENTCOLOR,COMMENTCOLOR);
        if( m_Properties.getProperty(PropertiesManager.P_LITERALCOLOR) == null ) m_Properties.setProperty(PropertiesManager.P_LITERALCOLOR,LITERALCOLOR);
        if( m_Properties.getProperty(PropertiesManager.P_INVALIDCOLOR) == null ) m_Properties.setProperty(PropertiesManager.P_INVALIDCOLOR,INVALIDCOLOR);
        if( m_Properties.getProperty(PropertiesManager.P_LINENUMBER) == null ) m_Properties.setProperty(PropertiesManager.P_LINENUMBER,LINENUMBER);

        PropertiesManager.MICCOLOR[0] = Integer.parseInt( m_Properties.getProperty(PropertiesManager.P_COMMENTCOLOR) ); 
        PropertiesManager.MICCOLOR[5] = Integer.parseInt( m_Properties.getProperty(PropertiesManager.P_LITERALCOLOR) );
        PropertiesManager.MICCOLOR[9] = Integer.parseInt( m_Properties.getProperty(PropertiesManager.P_INVALIDCOLOR) );
               
        m_bLineNr = (Boolean.valueOf(m_Properties.getProperty(PropertiesManager.P_LINENUMBER))).booleanValue();
        
        
        XPOS = 23;
        YPOS = 43;
        XSIZE = 609;
        YSIZE = 538;
        
        // Font settings
        m_MenuItemFont	= new Font("Serif, SansSerif", Font.PLAIN, 11);
        m_MenuFont	= new Font("Serif, SansSerif", Font.PLAIN, 13);
    }

	/**
	 * Returns the font for the editor
	 */
	public Font getEditorFont()
    {
    	return new Font( m_Properties.getProperty(PropertiesManager.P_EDITORFONT), Font.PLAIN, Integer.parseInt( m_Properties.getProperty("EditorFontSize") ) );	
    }
    
    /**
	 * Returns the printer font
	 */
	public Font getPrinterFont()
    {
    	return new Font( m_Properties.getProperty(PropertiesManager.P_PRINTERFONT), Font.PLAIN, Integer.parseInt( m_Properties.getProperty("PrinterFontSize") ) );	
    }
    
    /**
	 * Returns the font for the menu items
	 */
    public Font getMenuItemFont()
    {
    	return m_MenuItemFont;	
    }
	/**
	 * Returns the font for the menu
	 */
    public Font getMenuFont()
    {
    	return m_MenuFont;	
    }
    
	/**
	 * returns the property
	 *
	 * @param: requested property
	 */
    public int getProperty(String key)
    {    	
    	return Integer.parseInt( m_Properties.getProperty(key) );	
    }
    
	/**
	 * Returns tru if Line Numbers are to be set, else false
	 */
	public boolean isLineNrEnabled()
    {
    	return m_bLineNr;
    }

    /**
	 * Stores the point size of the editor Font property
	 *
	 * @param : point size of the font
	 */
	public void setEditorFontSize(int size)
    {  	m_Properties.setProperty(PropertiesManager.P_EDITORFONTSIZE,String.valueOf(String.valueOf(size)) );    }
   
    /**
	 * Stores the editor font property
	 *
	 * @param: Font name
	 * @param: Font size
	 */
    public void setEditorFont(String font, int size)
    { 	
    	m_Properties.setProperty(PropertiesManager.P_EDITORFONT,font);
    	m_Properties.setProperty(PropertiesManager.P_EDITORFONTSIZE,String.valueOf(size) );
	}

	/**
	 * Stores the printer font property
	 *
	 * @param: Font name
	 * @param: Font size
	 */
    public void setPrinterFont(String font, int size)
    { 	
    	m_Properties.setProperty(PropertiesManager.P_PRINTERFONT,font);
    	m_Properties.setProperty(PropertiesManager.P_PRINTERFONTSIZE,String.valueOf(size) );
	}

	/**
	 * Sets the point size of the menu item Font
	 *
	 * @param : point size of the font
	 */
    public void setMenuItemFontSize(int size)
    {
    	m_MenuItemFont = new Font(m_MenuItemFont.getName(), m_MenuItemFont.getStyle(), size);
    }

	/**
	 * Sets the menu item font
	 */
    public void setMenuItemFont(Font f)
    { 	m_MenuItemFont = f;				}

	/**
	 * Sets the point size of the menu Font
	 *
	 * @param : point size of the font
	 */
    public void setMenuFontSize(int size)
    {	m_MenuFont = new Font(m_MenuFont.getName(), m_MenuFont.getStyle(), size);    }

	/**
	 * Sets the menu font
	 */
    public void setMenuFont(Font f)
    { 	m_MenuFont = f;					}


	/**
	 * Stores the color for the text comment property
	 */
	public void setCommentColor( int c )
	{
		PropertiesManager.MICCOLOR[0] = c;
		m_Properties.setProperty("CommentColor",String.valueOf(c));
	}
	
	/**
	 * Stores the color for the text comment property
	 */
	public void setLiteralColor( int c )
	{
		PropertiesManager.MICCOLOR[5] = c;
		m_Properties.setProperty("LiteralColor",String.valueOf(c));
	}
	
	/**
	 * Stores the color for the invalid text property
	 */
	public void setInvalidColor( int c )
	{
		PropertiesManager.MICCOLOR[9] = c;
		m_Properties.setProperty("InvalidColor",String.valueOf(c));
	}

	/**
	 * Stores linenumber property
	 *
	 * @param: true or false
	 */
	public void setLineNr(boolean bState)
	{
		m_bLineNr = bState;
		m_Properties.setProperty(PropertiesManager.P_LINENUMBER, String.valueOf(bState));
	}
	
	
	/**
	 * Sets default properties
	 */
	public void setDefaults( )
	{
		m_Properties.setProperty(PropertiesManager.P_EDITORFONT,EDITORFONT);
        m_Properties.setProperty(PropertiesManager.P_EDITORFONTSIZE,EDITORFONTSIZE);
        m_Properties.setProperty(PropertiesManager.P_COMMENTCOLOR,COMMENTCOLOR);
        m_Properties.setProperty(PropertiesManager.P_LITERALCOLOR,LITERALCOLOR);
        m_Properties.setProperty(PropertiesManager.P_INVALIDCOLOR,INVALIDCOLOR);
        m_Properties.setProperty(PropertiesManager.P_LINENUMBER,LINENUMBER);
           
        PropertiesManager.MICCOLOR[0] = Integer.parseInt( m_Properties.getProperty(PropertiesManager.P_COMMENTCOLOR) ); 
        PropertiesManager.MICCOLOR[5] = Integer.parseInt( m_Properties.getProperty(PropertiesManager.P_LITERALCOLOR) );
        PropertiesManager.MICCOLOR[9] = Integer.parseInt( m_Properties.getProperty(PropertiesManager.P_INVALIDCOLOR) );
    }
		
	
//************************************** Load & Store Properties ***************************************	

	/**
	 * Load property settings to propertiesmanager 
	 *
	 * @param: property file
	 */
	public void loadProperties(String file )
	{
		//load config-file if exists
        m_propertiesFile = new File(file);
        if( m_propertiesFile.exists() && m_propertiesFile.canRead() )
        {
        	try
        	{
        		m_Properties.load( new FileInputStream( m_propertiesFile ) );
        	}
        	catch( Exception e )
        	{
        		System.out.println("File not found");
        	}
        }
    }
         
	/**
	 * Store settings from propertiesmanager to property file
	 */
	public void storeProperties()
	{
	 	try
    	{
    		m_Properties.store( new FileOutputStream( m_propertiesFile ), "MIC1 Simulator config file" );
    	}
    	catch( Exception e )
    	{ 	}
    }		
}