/* 
*  ALU.java
*
*  mic1 microarchitecture simulator 
*  Copyright (C) 1999, Prentice-Hall, Inc. 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  You should have received a copy of the GNU General Public License along with 
*  this program; if not, write to: 
* 
	 Free Software Foundation, Inc. 
	 59 Temple Place - Suite 330 
	 Boston, MA 02111-1307, USA. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
	 http://www.gnu.org/copyleft/gpl.html
* 
*/ 
package mic1.ijvm;
import java.awt.*;

/**
* Arithmetic logic unit.
*
* @author 
*   Dan Stone (<a href="mailto:dans@ontko.com"><i>dans@ontko.com</i></a>),
*   Ray Ontko & Co,
*   Richmond, Indiana, US
*/
/**
 * Modification History:
 *
 * Name             Date       Comment
 * ---------------- ---------- ----------------------------------------
 * W.D. de Witte    01.12.2000 Rewritten and Removed GUI stuff
 *                            
 */
public class ALU
{
	private Bus a_bus = null;
	private Bus b_bus = null;
	private Bus out = null;
  
  	private ControlLine alu_cl = null;
	private ControlLine n_cl = null;
	private ControlLine z_cl = null;

  	int result;

  	public ALU(Bus a_bus, Bus b_bus, Bus out, 
	  ControlLine alu_cl, ControlLine n_cl,
	  ControlLine z_cl) 
	{
		this.a_bus = a_bus;
		this.b_bus = b_bus;
		this.out = out;
		this.alu_cl = alu_cl;
		this.n_cl = n_cl;
		this.z_cl = z_cl;
  	}
    
    public boolean poke() 
  	{
		int a = 0; 
		int b = 0; 
		result = 0;	
		boolean value[] = alu_cl.getValue();
		if (value[MIR.ENA])
	  		a = a_bus.getValue();
		if (value[MIR.ENB])
	  		b = b_bus.getValue();
		if (value[MIR.INVA])
	  		a = ~ a;
		if (value[MIR.F0]) 
		{
	  		if (value[MIR.F1])
				result = a + b;
	  		else
				result = ~ b;
		}
		else 
		{
	  		if (value[MIR.F1])
				result = (a | b);
	  		else
				result = (a & b);    
		}
		if (value[MIR.INC])
	  		result++;

		out.setValue(result);
		boolean z_ar[] = {result == 0};
		boolean n_ar[] = {result < 0};
		
		z_cl.setValue(z_ar);
		n_cl.setValue(n_ar);
		return true;
  	}
	boolean getZ() {return z_cl.getValue()[0];}
	boolean getN()	{return n_cl.getValue()[0];}	
	int getA() {return a_bus.getValue();}
	int getB()	{return b_bus.getValue();}
	int getOut() {return out.getValue();}
	
  	public String toString() 
  	{  		
		boolean value[] = alu_cl.getValue();
		String s = new String();
		if (value[MIR.INVA])
	  		s += "NOT ";
		if (value[MIR.ENA])
	  		s += "A ";
		else
	  		s += "0 ";
		if (value[MIR.F0]) 
		{
	  		if (value[MIR.F1])
				s += "+ ";
	  		else 
				s = "NOT ";
		}
		else 
		{
	  		if (value[MIR.F1])
				s += "OR ";
	  		else 
				s += "AND ";
		}
		if (value[MIR.ENB])
	  		s += "B ";
		else
	  		s += "0 ";
		if (value[MIR.INC])
	  		s += "+ 1 ";
		s += "= 0x" + Integer.toHexString(result).toUpperCase();
		return s;
  	}
}
