/*
*
*  IJVMLoader.java
*
*  mic1 microarchitecture simulator 
*  Copyright (C) 1999, Prentice-Hall, Inc. 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  You should have received a copy of the GNU General Public License along with 
*  this program; if not, write to: 
* 
*    Free Software Foundation, Inc. 
*    59 Temple Place - Suite 330 
*    Boston, MA 02111-1307, USA. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 
package mic1.ijvm;
import java.io.*;
import java.util.*; // KLOS

/**
* Loads an IJVM program into memory.
*
* @author 
*   Dan Stone (<a href="mailto:dans@ontko.com"><i>dans@ontko.com</i></a>),
*   Ray Ontko & Co,
*   Richmond, Indiana, US
*/

/*

Modification History

Name             Date       Comment
---------------- ---------- ----------------------------------------
Richard Klos     11-11-2000 Added attribute lineTable.
                            Modified constructor to read line
                            number table.
                            Modified method readBlock.
                            Added method loadTable.
                            Added method readInt (please forgive me).
*/

public class IJVMLoader implements Mic1Constants
{
	private BufferedInputStream in = null;
	private String filename;
	private String msg = null;
	private byte program[];
	protected Hashtable lineTable = null;
	private boolean eof;
	
	public IJVMLoader(String filename) throws IOException
	{
		this.filename = filename;
		eof = false;
		program = new byte[MEM_MAX];
		
		in = new BufferedInputStream(new FileInputStream(filename));

		if(validateFile())
		{	
			
			try
			{
				// Load first two blocks into program memory;
				// First block contains constants
				// Second block is the byte code.
				for(int i=0; i < 2; i++)
					readBlock(program);
					
				// Load data into line number table.
				loadTable();
			}
			catch (EOFException eofe)
			{
				eof = true;
				System.out.println("Unexpected end of file.");
			}
			catch (IOException ioe)
			{
				System.out.println(ioe);
			}
		}
		
/*
		System.out.println("Testing table.");
		for(int i=0; i < 2000; i++)
		{
			Object obj = lineTable.get(new Integer(i));
			if(obj != null)
			{
				int line = ((Integer)obj).intValue();
				System.out.println(i +" at line" + line);
			}
		}
*/
	}
	
	public String isValid()
	{
		return msg;
	}
	
	public String getFileName()
	{
		return filename;
	}
	
	public byte[] getProgram()
	{
		return program;
	}
	
	public Hashtable getLineNumberTable()
	{
		return lineTable;
	}
	
	private boolean validateFile()
	{
		try
		{
			if (in.read() != magic1 || in.read() != magic2 || in.read() != magic3 || in.read() != magic4)
			{
			    msg = "File error: invalid file format: " +filename;
			    return false;
			}
			else
			{
			    msg = null;
			    return true;
			}
		}
		catch (IOException e)
		{
			msg = "Error while reading file " + filename;
			return false;
		}
	}
	

	private void readBlock(byte [] destination) throws EOFException, IOException
	{
		int origin = readInt();
		int byte_count = readInt();
		
		for (int i = 0; i < byte_count; i++)
			destination[origin + i] = (byte)read();		
	}


	private void loadTable() throws EOFException, IOException
	{
		lineTable = null;
		int size = readInt();
		if(size < 0)
			return;
		lineTable = new Hashtable(size);
		
		for(int i=0; i < size; i++)
			lineTable.put(new Integer(readInt()), new Integer(readInt()));
	}
	
	private int readInt() throws IOException
	{
		return (read() << 24) + (read() << 16) +(read() << 8) + read();
	}	

	
	private int read() throws IOException
	{
		int i = in.read();
		if (i == -1)
			eof = true;
		return i;
	}
	
/*	public static void main(String args[]) throws Exception
	{
		IJVMLoader re = new IJVMLoader(args[0]);
	}
*/
}