package mic1.ijvm;
/*
 * @(#)KeyBuffer.java	1.0 06/Dec/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.00 06/Dec/2000
 * @Author  : Richard Klos
 *
 * Description : A characters queue. The queue can grow when needed.
 *               
 * 
 * Filter for file types
 * 
***  

To Do:
	
Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
Richard Klos     06.12.2000 Created

*/
import java.util.Vector;


public class KeyBuffer
{
	private Vector m_buffer;
	
	/**
	 * Constructs a new buffer with an initial
	 * size of 20.
	 */
	public KeyBuffer()
	{
		this(20);
	}
	

	/**
	 * Constructs a new buffer with a specified
	 * initial size.
	 */
	public KeyBuffer(int initialSize)
	{
		m_buffer = new Vector(initialSize);
	}
	
	
	/**
	 * Adds a character to the buffer.
	 */
	public void addChar(char c)
	{
		m_buffer.addElement(new Character(c));
	}
	
	
	/**
	 * Returns true if there are characters waiting in
	 * the buffer.
	 */
	public boolean hasMoreCharacters()
	{
		return m_buffer.size() != 0;
	}
	
	
	/**
	 * Removes the 'oldest' character from the buffer and
	 * returns it.
	 */
	public char getNextCharacter()
	{
		char c = peekNextCharacter();
		m_buffer.removeElementAt(0);
		return c;
	}
	
	
	/**
	 * Returns the 'oldest' character from the buffer, without
	 * removing it.
	 */
	public char peekNextCharacter()
	{
		return ((Character)m_buffer.elementAt(0)).charValue();
	}


	/**
	 * Clear the buffer.
	 */
	public void clear()
	{
		m_buffer.removeAllElements();
	}
}