/*
*
*  HighBit.java
*
*  mic1 microarchitecture simulator 
*  Copyright (C) 1999, Prentice-Hall, Inc. 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  You should have received a copy of the GNU General Public License along with 
*  this program; if not, write to: 
* 
*    Free Software Foundation, Inc. 
*    59 Temple Place - Suite 330 
*    Boston, MA 02111-1307, USA. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 
package mic1.ijvm;
import java.awt.*;

/**
* Memory address register.
*
* @author 
*   Dan Stone (<a href="mailto:dans@ontko.com"><i>dans@ontko.com</i></a>),
*   Ray Ontko & Co,
*   Richmond, Indiana, US
*/
/**
 * Modification History:
 *
 * Name             Date       Comment
 * ---------------- ---------- ----------------------------------------
 * W.D. de Witte    01.12.2000 Rewritten and Removed GUI stuff
 *                            
 */
public class MAR
{
	private int value;
  	private Bus in = null;
    private Bus address_bus = null;
  	private ControlLine store_cl = null;
    private ControlLine mem_cl = null;
    private String sDebugText = new String();

  	public MAR(Bus in, Bus address_bus, ControlLine store_cl, ControlLine mem_cl) 
  	{
    	this.in = in;
    	this.address_bus = address_bus;
    	this.store_cl = store_cl;
    	this.mem_cl = mem_cl;
  	}

  	public void store() 
  	{
    	if (store_cl.getValue()[0]) 
    	{
      		value = in.getValue();
      		sDebugText = "MAR: Store 0x" + toString();
    	}
  	}

  	public void mem() 
  	{
  		if (mem_cl.getValue()[MIR.READ])
  		{
      		address_bus.setValue(value);
      		sDebugText = "MAR: Read from word 0x" + toString();
    	}

    	if (mem_cl.getValue()[MIR.WRITE]) 
    	{
      		address_bus.setValue(value);
			sDebugText = "MAR: Write to word 0x" + toString();
		}
  	}
  	public String getDebugString(){return sDebugText;}

	public int getValue()
	{
		return value;
	}
	
	public String toString()
	{
		return new String("0x" + Integer.toHexString(value).toUpperCase());
	}

  	public void forceValue(int value) 
  	{
    	value = value;
  	}
}