/*
*  MBR.java
*
*  mic1 microarchitecture simulator 
*  Copyright (C) 1999, Prentice-Hall, Inc. 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  You should have received a copy of the GNU General Public License along with 
*  this program; if not, write to: 
* 
*    Free Software Foundation, Inc. 
*    59 Temple Place - Suite 330 
*    Boston, MA 02111-1307, USA. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/
package mic1.ijvm;
import java.awt.*;

/**
* Register that stores bytes fetched from memory.
*
* @author 
*   Dan Stone (<a href="mailto:dans@ontko.com"><i>dans@ontko.com</i></a>),
*   Ray Ontko & Co,
*   Richmond, Indiana, US
*/
/**
 * Modification History:
 *
 * Name             Date       Comment
 * ---------------- ---------- ----------------------------------------
 * W.D. de Witte    01.12.2000 Rewritten and Removed GUI stuff
 *                            
 */
public class MBR
{
	private int value, next_val;
  	
  	private Bus in = null;
	private Bus out = null;
  	
  	private ControlLine memory_cl = null;
    private ControlLine store_cl = null;
    private ControlLine mbr_cl = null;
    private ControlLine mbru_cl = null;
  
  	private IntControlLine o_cl = null;
  
  	private boolean fetch = false;
  	private String sDebugText = "";

  	public MBR(Bus in, Bus out, ControlLine memory_cl, ControlLine mbr_cl,
	     ControlLine mbru_cl, IntControlLine o_cl) 
	{
    	this.in = in;
    	this.out = out;
    	this.store_cl = store_cl;
    	this.mbr_cl = mbr_cl;
    	this.mbru_cl = mbru_cl;
    	this.o_cl = o_cl;
    	this.memory_cl = memory_cl;
  	}

  	public void store() 
  	{
  		if (memory_cl.getValue()[0]) 
  		{
      		value = in.getValue();
      		o_cl.setValue(value & 255);
      		sDebugText = "MBR: Store 0x" + toString();
		}
  	}

  	public void put() 
  	{
    	if (mbr_cl.getValue()[0]) 
    	{
      		sDebugText = "MBR: Put 0x" + toString();
      		if ((value & 128) == 0)
				out.setValue(value & 255);
      		else 
				out.setValue(value | -256);
    	}
    	else if (mbru_cl.getValue()[0]) 
    	{
    		sDebugText = "MBR: Put 0x" + toString();
      		out.setValue(value & 255);
    	}
  	}

  	public void forceValue(int value) 
  	{
	    this.value = value;
  	}
	
	public String getDebugString(){return sDebugText;}
	
	public int getValue() {return value;}
	
	public String toString()
	{
		return new String("0x" + Integer.toHexString(value).toUpperCase());
	}
}
