package mic1.ijvm;
//import mic1.ijvm.*;

/*

Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
WDW              xx.xx.2000 Created
Richard Klos     06.12.2000 Added attribute m_keyBuffer
                            Added method getKeyBuffer()
                            

*/


import java.io.*;
import java.util.*;

public class Mic1Kernel implements Mic1Constants
{
	private KeyBuffer m_keyBuffer;
	
	private int cycle_count;
	private boolean null_array[] = {false, false, false, false, false, false};
	private boolean true_array[] = {true};
	protected boolean mp_loaded = false;
	
	// Components 
	protected ControlStore control_store = null;
	protected MainMemory main_memory = null;
	protected ALU alu = null;
	protected Shifter shifter = null;
	protected Decoder decoder = null;
	protected O o = null;
	protected HighBit high_bit = null;
	
	// Registers
	protected Register sp = null;
	protected Register lv = null;
	protected Register cpp = null;
	protected Register tos = null;
	protected Register opc = null;
	protected Register h = null;
	
	protected MAR mar = null;
	protected PC pc = null;
	protected MDR mdr = null;
	protected MBR mbr = null;
	protected MPC mpc = null;
	protected MIR mir = null;
	
	// Busses
	protected Bus byte_address_bus = null;
	protected Bus word_address_bus = null;
	protected Bus byte_data_bus = null;
	protected Bus word_data_bus = null;
	protected Bus a_bus = null;
	protected Bus b_bus = null;
	protected Bus c_bus = null;
	protected Bus alu_bus = null;
	protected ObjectBus mir_bus = null;
	
	// Component control lines
	protected IntControlLine control_store_cl = null;
	protected IntControlLine decoder_cl = null;
	protected IntControlLine o_addr_cl = null;  // Carries next microinstruction address to O
	protected IntControlLine o_mbr_cl = null;   // Carries MBR value to O
	protected IntControlLine mpc_cl = null;     // Carries O value to MPC
	
	protected ControlLine memory_cl = null;
	protected ControlLine alu_cl = null;
	protected ControlLine shifter_cl = null;
	protected ControlLine o_jmpc_cl = null;    // Carries JMPC from MIR to O
	protected ControlLine jam_cl = null;       // Carries JAMN/JAMZ from MIR to High bit
	protected ControlLine high_bit_cl = null;  // Carries bit from High bit to MPC
	protected ControlLine n_cl = null;         // Negative flag from ALU to High bit
	protected ControlLine z_cl = null;         // Zero flag from ALU to High bit
	
	// Register control lines
	//   Each register has 2 control lines, one to indicate whether to store the value
	//   on the IN bus, the other to indicate whether to put its value onto the OUT
	//   bus.  Two control lines are used instead of one because the control signals
	//   come from two different sources--STORE from the MIR, and PUT from the decoder--
	//   and the two operations occur at different times in the clock cycle
	protected ControlLine mar_store_cl = null;
	protected ControlLine mar_put_cl = null;
	protected ControlLine mdr_store_cl = null;
	protected ControlLine mdr_put_cl = null;
	protected ControlLine mdr_mem_cl = null;
	protected ControlLine pc_store_cl = null;
	protected ControlLine pc_put_cl = null;
	protected ControlLine mbr_store_cl = null;
	protected ControlLine mbr_put_cl = null;  // MBR stores only on a memory fetch, so it does not have a 
	protected ControlLine mbru_put_cl = null; // store control line.  It has 2 put control lines, signed and unsigned
	protected ControlLine sp_store_cl = null;
	protected ControlLine sp_put_cl = null;
	protected ControlLine lv_store_cl = null;
	protected ControlLine lv_put_cl = null;
	protected ControlLine cpp_store_cl = null;
	protected ControlLine cpp_put_cl = null;
	protected ControlLine tos_store_cl = null;
	protected ControlLine tos_put_cl = null;
	protected ControlLine opc_store_cl = null;
	protected ControlLine opc_put_cl = null;
	protected ControlLine h_store_cl = null;
	protected ControlLine h_put_cl = null;  // In the dual-bus data path, this control line will always be asserted
	
	/////////////////////////////////////////////////////////////////////
	// Setting register values
	/////////////////////////////////////////////////////////////////////	
	public Mic1Kernel()
	{

		// First, create a buffer for characters coming from the console.
		m_keyBuffer = new KeyBuffer(25);
		// Next, create busses and control lines		
		byte_address_bus = new Bus();
		word_address_bus = new Bus();
		byte_data_bus = new Bus();
		word_data_bus = new Bus();
		a_bus = new Bus();
		b_bus = new Bus();
		c_bus = new Bus();
		alu_bus = new Bus();
		mir_bus = new ObjectBus();
		
		// Control lines
		control_store_cl = new IntControlLine();
		memory_cl = new ControlLine();
		alu_cl = new ControlLine();
		shifter_cl = new ControlLine();
		decoder_cl = new IntControlLine();
		o_mbr_cl = new IntControlLine();
		o_addr_cl = new IntControlLine();
		o_jmpc_cl = new ControlLine();
		mpc_cl = new IntControlLine();
		jam_cl = new ControlLine();
		high_bit_cl = new ControlLine();
		n_cl = new ControlLine();
		z_cl = new ControlLine();
		
		mar_store_cl = new ControlLine();
		mar_put_cl = new ControlLine();
		mdr_store_cl = new ControlLine();
		mdr_put_cl = new ControlLine();
		mdr_mem_cl = new ControlLine();
		pc_store_cl = new ControlLine();
		pc_put_cl = new ControlLine();
		mbr_store_cl = new ControlLine();
		mbr_put_cl = new ControlLine();
		mbru_put_cl = new ControlLine();
		sp_store_cl = new ControlLine();
		sp_put_cl = new ControlLine();
		lv_store_cl = new ControlLine();
		lv_put_cl = new ControlLine();
		cpp_store_cl = new ControlLine();
		cpp_put_cl = new ControlLine();
		tos_store_cl = new ControlLine();
		tos_put_cl = new ControlLine();
		opc_store_cl = new ControlLine();
		opc_put_cl = new ControlLine();
		h_store_cl = new ControlLine();
		h_put_cl = new ControlLine();
		h_put_cl.setValue(true_array);
		
		// Next, create components and registers
		// Registers
		mar = new MAR(c_bus, word_address_bus, mar_store_cl, memory_cl);
		sp = new Register(c_bus, b_bus, "SP", sp_store_cl, sp_put_cl);
		lv = new Register(c_bus, b_bus, "LV", lv_store_cl, lv_put_cl);
		cpp = new Register(c_bus, b_bus, "CPP", cpp_store_cl, cpp_put_cl);
		
		tos = new Register(c_bus, b_bus, "TOS", tos_store_cl, tos_put_cl);
		opc = new Register(c_bus, b_bus, "OPC", opc_store_cl, opc_put_cl);
		h = new Register(c_bus, a_bus, "H", h_store_cl, h_put_cl);
		pc = new PC(c_bus, b_bus, byte_address_bus, pc_store_cl, pc_put_cl, memory_cl);
		mdr = new MDR(c_bus, b_bus, word_data_bus, mdr_store_cl, mdr_put_cl, mdr_mem_cl, memory_cl);
		mbr = new MBR(byte_data_bus, b_bus, mbr_store_cl, mbr_put_cl, mbru_put_cl, o_mbr_cl);
		mir = new MIR(	o_addr_cl, o_jmpc_cl, jam_cl, shifter_cl, alu_cl, mar_store_cl, mdr_store_cl,
			  			pc_store_cl, sp_store_cl, lv_store_cl, cpp_store_cl, tos_store_cl, opc_store_cl, 
			  			h_store_cl, memory_cl, decoder_cl, mir_bus);
		
		// Components
		control_store = new ControlStore(control_store_cl, mir_bus);
		mpc = new MPC(mpc_cl, high_bit_cl, control_store_cl, control_store);
		main_memory = new MainMemory(this, byte_address_bus, word_address_bus, //KLOS
					 byte_data_bus, word_data_bus, memory_cl, mbr_store_cl, mdr_mem_cl, mdr);
		alu = new ALU(a_bus, b_bus, alu_bus, alu_cl, n_cl, z_cl);
		shifter = new Shifter(alu_bus, c_bus, shifter_cl);
		decoder = new Decoder(decoder_cl, mdr_put_cl, pc_put_cl, mbr_put_cl, mbru_put_cl,
				  sp_put_cl, lv_put_cl, cpp_put_cl, tos_put_cl, opc_put_cl);
		o = new O(o_addr_cl, o_mbr_cl, o_jmpc_cl, mpc_cl);
		high_bit = new HighBit(n_cl, z_cl, jam_cl, high_bit_cl);
	}


	public KeyBuffer getKeyBuffer()
	{
		return m_keyBuffer;
	}
	
	public void writeCharToConsole(char c){}

	/**
	 * Loads the micro program in memory.
	 *
	 * @param : filepath of the program
	 * @exception : IOException for file access;
	 */	
	public void loadMicroprogram(String filename) throws Exception
	{
		Mic1Instruction microprogram[] = new Mic1Instruction[612];

		InputStream in = new FileInputStream(filename);
		boolean eof = false;
		if (in.read() != mic1_magic1 ||
			in.read() != mic1_magic2 ||
      		in.read() != mic1_magic3 ||
      		in.read() != mic1_magic4) 
      	{
    		mp_loaded = false;
  		}
  		else 
  		{
    		int i = 0;
    		while (!eof) 
    		{
    			microprogram[i] = new Mic1Instruction();
  				eof = (microprogram[i].read(in) == -1);
  				i++;
    		}
    		setControlStore(microprogram);
    		mp_loaded = true;
    		reset();
    		return;
  		}
      	mp_loaded = false;
  	}
  	
  	protected int getProgramCounterValue()
  	{
  		return pc != null? pc.getValue():-1;
  	}

	
	protected void setMemory(byte[] bytes)
	{
		main_memory.setMemory(bytes);
	}
	
	protected void setControlStore(Mic1Instruction instructions[]) 
	{
		control_store.setStore(instructions);
	}
	
	/** 
	 * Returns true when the memory contains data
	 *
	 * @return : boolean!
	 */	
	public boolean validMemory ()
	{
		return main_memory.validMemory();
	}

	public int getCycleCount()
	{
		return cycle_count;
	}
	
	public boolean cycle ()
	{
		cycle_count++;		
		clearControlLines();
    	// Drive next microinstruction onto control lines
    	control_store.poke();
    	mir.poke();

    	// Data path cycle
    	decoder.poke();
    	mdr.put();
    	pc.put();
    	mbr.put();
    	sp.put();
    	lv.put();
    	cpp.put();
    	tos.put();
    	opc.put();
    	h.put();
    	alu.poke();
    	shifter.poke();
	
	    mar.store();
    	mdr.store();
    	pc.store();
    	sp.store();
    	lv.store();
    	cpp.store();
    	tos.store();
    	opc.store();
    	h.store();
	
    	pc.mem();
    	mar.mem();
    	mdr.mem();
    	main_memory.poke();
    	mbr.store();
	
    	// Control path cycle
    	high_bit.poke();
    	o.poke();
    	
    	return mpc.poke();
	}
	

	
	protected void clearControlLines() 
	{
		mbr_store_cl.setValue(null_array);
		mdr_mem_cl.setValue(null_array);
		memory_cl.setValue(null_array);
		alu_cl.setValue(null_array);
		shifter_cl.setValue(null_array);
		o_jmpc_cl.setValue(null_array);
		jam_cl.setValue(null_array);
		high_bit_cl.setValue(null_array);
		n_cl.setValue(null_array);
		z_cl.setValue(null_array);
		mar_store_cl.setValue(null_array);
		mar_put_cl.setValue(null_array); 
		mdr_store_cl.setValue(null_array);
		mdr_put_cl.setValue(null_array);
		pc_store_cl.setValue(null_array);
		pc_put_cl.setValue(null_array);
		mbr_put_cl.setValue(null_array); 
		mbru_put_cl.setValue(null_array);
		sp_store_cl.setValue(null_array);
		sp_put_cl.setValue(null_array);
		lv_store_cl.setValue(null_array);
		lv_put_cl.setValue(null_array);
		cpp_store_cl.setValue(null_array);
		cpp_put_cl.setValue(null_array); 
		tos_store_cl.setValue(null_array);
		tos_put_cl.setValue(null_array);
		opc_store_cl.setValue(null_array);
		opc_put_cl.setValue(null_array);
		h_store_cl.setValue(null_array);
	}

	/** 
	 * Resets the kernel. The program loaded persists in memory
	 */	
	public void reset() throws Exception
	{
		if(mp_loaded)
		{	
			cycle_count = 0;
			control_store_cl.setValue(0);
			clearControlLines();
			mpc.forceValue(0);
			pc.forceValue(-1);
			mbr.forceValue(0);
			mar.forceValue(0);
			mdr.forceValue(0);
			opc.forceValue(0);
			tos.forceValue(0);
			h.forceValue(0);
			mir.forceValue(null);
			sp.forceValue(SP);
			lv.forceValue(LV);
			cpp.forceValue(CPP);
			m_keyBuffer.clear();	//KLOS
			main_memory.reset();
			
			control_store_cl.setValue(0);
			decoder_cl.setValue(0);
			o_mbr_cl.setValue(0); 
			o_addr_cl.setValue(0); 
			mpc_cl.setValue(0); 
			
			byte_address_bus.setValue(0);
			word_address_bus.setValue(0);
			byte_data_bus.setValue(0);
			word_data_bus.setValue(0); 
			a_bus.setValue(0);
			b_bus.setValue(0);
			c_bus.setValue(0);
			alu_bus.setValue(0);
			mp_loaded = false;
		}
	}
	protected void forceReset()
	{
		mp_loaded = true;
		try
		{
			reset();
		}
		catch(Exception e){}
	}
}
