package mic1.ijvm;
/*
 * @(#)Mic1Output.java	1.0 02/Nov/2000
 * 
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version. 
 *
 * @Version : 1.00 02/Nov/2000
 * @Author  : WDW
 *
 * Description : 
 * 
 * Output interface for catching messages from MIC1 machine.
 * 

Modification History:

Name             Date       Comment
---------------- ---------- ----------------------------------------
WDW              02.11.2000 Created
Richard Klos     06.12.2000 Added method writeCharToConsole(char)
Richard Klos     16.12.2000 Changed method name writeCharToConsole
                            to OnCharOut.
Richard Klos     16.01.2001 Removed all hex-string as parameters.
 */        
   
public interface Mic1OutputInterface
{
    /** 
     * Invoked by the machine just before starting the thread.
     *
     */	
	public void OnRun();
	
    /** 
     * The machine stopped at a breakpoint position. This function
     * is called after OnStop();
     * 
     * @param : the linenumber of the breakpoint
     */		
	public void OnReachedBreakPoint(int nLineNumber);

    /** 
     * The machine thread has been stopped 
     * 
     * @param : the linenumber of the current instruction
     * @param : the current cycle (step count)
     */			
	public void OnStop(int nLineNumber, int nCycleCount);
	
    /** 
     * The machine is going to execute the current cycle
     * 
     * @param : the linenumber of the current instruction.
     * @param : the current cycle (step count)
     */				
	public void OnStep(int nLineNumber, int nCycleCount);
	
    /** 
     * The value of the MAR has been changed
     * 
     * @param : value of the register
     */		
	public void OnChangedMAR(int nValue);
	
    /** 
     * The value of the MDR has been changed
     * 
     * @param : value of the register
     */			
	public void OnChangedMDR(int nValue);
	
    /** 
     * The value of the PC has been changed
     * 
     * @param : value of the register
     */				
	public void OnChangedPC(int nValue);
	
    /** 
     * The value of the MBR has been changed
     * 
     * @param : value of the register
     */					
	public void OnChangedMBR(int nValue);
	
    /** 
     * The value of the LV has been changed
     * 
     * @param : value of the register
     */						
	public void OnChangedLV(int nValue);
	
    /** 
     * The value of the CPP has been changed
     * 
     * @param : value of the register
     */							
	public void OnChangedCPP(int nValue);
	
    /** 
     * The value of the TOS has been changed
     * 
     * @param : value of the register
     */								
	public void OnChangedTOS(int nValue);
	
    /** 
     * The value of the OPC has been changed
     * 
     * @param : value of the register
     */									
	public void OnChangedOPC(int nValue);
	
    /** 
     * The value of the H has been changed
     * 
     * @param : value of the register
     */										
	public void OnChangedH(int nValue);
	
    /** 
     * The value of the MIR has been changed
     * 
     * @param : string representation of current micro-instruction
     * @param : string representation of next micro-instruction
     */										
	public void OnChangedMIR(String sValue, String sNextValue);
	
    /** 
     * The value of the MPC has been changed
     * 
     * @param : value of the register
     */											
	public void OnChangedMPC(int nValue);
	
    /** 
     * The values of the ALU has been changed
     * 
     * @param : inputline a
     * @param : inputline b
     * @param : outputline
     */												
	public void OnChangedALU(boolean n, boolean z,int out);

    /** 
     * The stack pointer
     * 
     * @param : value of the stackpointer
     */												
	public void OnChangedSP(int sValue);

    /** 
     * The standard output of the machine
     * 
     * @param : output text
     */											
	public void OnChangedStandardOut(String sText);
	
    /** 
     * The debug output of the machine
     * 
     * @param : output text
     */											
	public void OnChangedDebugOut(String sText);
	
   /** 
     * Invoked by the machine when it sends a character
     * to the console.
     * 
     * @param : a character.
     */
	public void OnCharOut(char charOut);
}

/*EOF*/