/**
 * Modification History:
 *
 * Name             Date       Comment
 * ---------------- ---------- ----------------------------------------
 * WDW              02.11.2000 Created
 *
 *
 */
/**
* Thread which allows mic1 to continuously cycle.  Without running
* the cycles in a background thread, the simulator would be unable to 
* respond to the Mic1Interface
*/
package mic1.ijvm;

public class PulseGenerator implements Runnable
{
	protected Thread m_Thread = null;
	protected boolean m_bSuspend = false;
	protected boolean m_bCompleted = false;
	protected boolean m_bRunning = false;
	protected PulseConsumer m_PulseConsumer = null;
	protected int m_nSleep;

	//**************************************************************	
	public PulseGenerator(PulseConsumer p, int nSleep)
	{
		m_PulseConsumer = p;
		m_nSleep = nSleep;
	}
		
		
	//**************************************************************		
	public void run()
	{
		m_PulseConsumer.OnStart();
		while(m_bRunning)
		{
			while(m_bSuspend && m_bRunning)
			{
				synchronized(this)
				{
					try{wait();}catch(InterruptedException e){}
				}
			}
			try
			{
				m_bRunning = m_PulseConsumer.OnClock() && m_bRunning;
				if(m_nSleep > 0)
					Thread.sleep(m_nSleep);
			}
			catch(Exception e)
			{
				m_bRunning = false;
				System.err.println(e);
			}
		}
		synchronized(this)
		{
            m_bCompleted = true;
            notifyAll();
        }
        m_PulseConsumer.OnStop();
	}
	
	
	//**************************************************************	
	public void startThread()
	{
		if(m_bRunning || m_PulseConsumer == null)
			return;
		m_Thread = new Thread(this);
		m_bRunning = true;
		m_Thread.start();
	}
 
 
	//************************************************************** 
	public synchronized void suspendThread()
	{
		m_bSuspend = true;
	}



	//**************************************************************
	public synchronized void resumeThread ()
	{
		if(m_bSuspend)
		{
			m_bSuspend = false;
			notify();	
		}
	}
	
	
	//**************************************************************
	public synchronized boolean isSuspended() { return m_bSuspend; }
	public synchronized boolean isRunning() { return m_bRunning; }
		
	
	//**************************************************************	
	public synchronized void stopThread ()
	{
		m_bRunning = false;
        while(!m_bCompleted) 
        {
            try {wait();} catch (InterruptedException exc) {}
        }		
	}
}